// 		Dynamic Statistics.cpp
//		Originally by Jordan Edgecombe
//		Chris Briglio

#define NOMINMAX

#include "stdafx.h"

#include <math.h>

#include "convert.h"

#include <boost/graph/adjacency_list.hpp>

#include <boost/graph/connected_components.hpp>

#include <boost\graph\betweenness_centrality.hpp>

#include <limits>


 //------------------------------------------------------------------------------DYNAMIC STATISTICS-----------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


using namespace boost;


/*
	DynamicStatistics function:
		Takes output file generated from Location Generator function and creates statistics from it using Boost Library.
		Statistics are stored in .txt file if the form: " *outputfilename* (*statistictype*).txt ".
		Statistics created:
			- average neighbours
			- most neighbours
			- average path length
			- diameter
			- components
			- node centrality
			- links
			- coverage
			- surface graph
			- neighbourhood instablility
			- average instability
			- clustering coefficient
			- average clustering
			- predictability
*/
int DynamicStatistics(std::string infile,double runtime,bool run)
{

	//----------------------------------------------------------VARIABLES----------------------------------------------------------//

	//-----------------------------------------------------------------------------------------------------------------------------//

	num_nodes = 0;

	int node_check = 1;

	int hashtag_check = 1;

	int refresh_interval = 0; 

	int transmission_range = 0;

	int x_axis = 0;

	int y_axis = 0;

	int total_time = 0;

	int end_index = 0;

	std::ifstream myinfile;

	std::string read_line = "";

	std::string graphviz = "";

	double time_percent = 0.0;

	int graphvizTime = 0;


	//----------------------------------------------------------FILE READ----------------------------------------------------------//

	//-----------------------------------------------------------------------------------------------------------------------------//

do		//loop while user doesnt input 'EXIT'
{

	if (run != true)
	{

		std::cout << "\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*             Dynamic Statistics              *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*  Please enter the path to the output file   *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*      Enter EXIT to exit to main menu        *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\n\n";
          
		std::cout << "\t                 >> "; std::getline (std::cin,infilename,'\n');   	// Prompts for the output file from Location Generator

		std::cout << "\n";

	}
	else
		infilename = infile;

	exitmask = infilename;

	to_upper(exitmask);

	if (exitmask != "EXIT")
	{

		myinfile.open(infilename);


		if (!myinfile.is_open()) 
		{

			std::cout << "Unable to open file: " << infilename <<"\n";

			std::system("PAUSE");

			std::system("CLS");

		}
		else
		{


			//Reads the total time until the network is static from the file

			std::cout << "Getting scenario run time...\n";

			while (myinfile.eof() == 0) 
			{

				std::getline(myinfile,read_line);

			} //end while


			total_time = convertToInterger(read_line);

			myinfile.close();


			//double run_time = -1;

			double run_time = -1;

			if (run != true)
			{

				//asks for how long the statistics should be gathered

				//this will be rounded down to the closest integer

				std::string runcheck = "";
				bool correct = true;

				std::system("CLS");

				do 
				{
					correct = true;
				
					std::cout << "\n";
					std::cout << "\t***********************************************\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*             Dynamic Statistics              *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*      Please enter the desired run time      *\n";
					std::cout << "\t*  (Warning: Run times greater than 250,000   *\n";
					std::cout << "\t*         may cause program to crash)         *\n";
					std::cout << "\t*        <enter 0 to run until static>        *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*       Enter EXIT to exit to main menu       *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t***********************************************\n";
					std::cout << "\n";

					std::cout << "\t    Network will be static in " << total_time << " seconds.\n\n";

					std::cout << "\t                 >> "; getline(std::cin,runcheck);

					exitmask = runcheck;

					to_upper(exitmask);

					if (exitmask != "EXIT")
					{

						if (runcheck.length() != 0)
						{
							for (size_t i = 0; i < runcheck.length() && correct == true;i++)
							{
								if (runcheck[i] > '9' || runcheck[i] < '0')
								{
									correct = false;
									std::system("CLS");
									std::cout << "Invalid input\n";
								}
							}
						}
						else
						{
							correct = false;
							std::system("CLS");
							std::cout << "Invalid input\n";
						}


						if (correct == true)
						{

							run_time = convertToDouble(runcheck);

							if (run_time < 0) 
							{
								std::system("CLS");
								std::cout << "Please enter a runtime greater than 0\n";
							} 
							else if (run_time > total_time) 
							{
								std::system("CLS");
								std::cout << "Please enter a runtime less than " << total_time << "\n";
							} 
							else if (run_time > 0) 
							{
								run_time = floor(run_time);
							} 
							else 
							{
								run_time = total_time;
							} //end if 

						}//end if

					}
					else
						return 0;

				}while (run_time < 0 || run_time > total_time);

			}
			else
				run_time = runtime;

			std::ofstream run_time_file;

			std::string run_time_filename = infilename;

			int run_txt_index = run_time_filename.find(" output\\") + 8;


			//writes the run-time to file

			//this is so the MATLAB statistics can be graphed for the approprate amount of time

			run_time_filename.erase(run_txt_index);

			run_time_filename.append("\\run_time.txt");

			run_time_file.open(run_time_filename);

			run_time_file << run_time;

			run_time_file.close();


			//reads the number of nodes, the refresh interval and the transmission range from the file

			myinfile.open(infilename);

			std::getline(myinfile,read_line);

			myinfile >> num_nodes;

			/*
				number of nodes must be more than 2 or will crash program at:

					else if (index[c] >= start_index[c+1] &&  c != num_nodes-1) 

				Because at variable definition:

					int* start_index = new int[num_nodes];

				if num_nodes is less than 2 then start_index[1] doesn't exist ( ie. lowest value is if c = 0 )

			*/
			if (num_nodes < 2)		//maybe
			{
				std::system("CLS");
				std::cout << "Invalid number of nodes\n";
				std::system("PAUSE");
				std::system("CLS");
				return 1;
			}

			myinfile >> refresh_interval;

			myinfile >> transmission_range;

			myinfile >> x_axis;

			myinfile >> y_axis;

			myinfile.ignore(20,'\n');	//possible cause of crash

			if (run != true)
			{

				std::system("CLS");

				do
				{
			
					std::cout << "\n";
					std::cout << "\t***********************************************\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*             Dynamic Statistics              *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*            Do you wish to make a            *\n";
					std::cout << "\t*       graphViz Visualization? (Yes/No)      *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t*       Enter EXIT to exit to main menu       *\n";
					std::cout << "\t*                                             *\n";
					std::cout << "\t***********************************************\n";
					std::cout << "\n\n";

					std::cout << "\t                   >> "; getline(std::cin,graphviz);

					to_upper(graphviz);

					if (graphviz != "YES" && graphviz != "NO" && graphviz != "EXIT")
					{
						std::system("CLS");
						std::cout << "Please answer Yes or No\n";
					}

				} while (graphviz != "YES" && graphviz != "NO" && graphviz != "EXIT");

				std::system("CLS");

				if (graphviz == "YES")
				{
					do
					{

						std::cout << "\n";
						std::cout << "\t***********************************************\n";
						std::cout << "\t*                                             *\n";
						std::cout << "\t*             Dynamic Statistics              *\n";
						std::cout << "\t*                                             *\n";
						std::cout << "\t*                                             *\n";
						std::cout << "\t*  When should the visualization be created?  *\n";
						std::cout << "\t*                                             *\n";
						std::cout << "\t***********************************************\n";
						std::cout << "\n";
					
						std::cout << "\t       (run time: " << run_time <<" refresh Interval: " << refresh_interval << ")\n\n";

						std::cout << "\t                  >> "; std::cin >> graphvizTime; std::cin.get();

						if (graphvizTime < 1)
						{
							std::system("CLS");
							std::cout << "Please enter a time greater than 1\n";
						} 
						else if (graphvizTime > run_time) 
						{
							std::system("CLS");
							std::cout << "Please enter a time less than " << run_time << "\n";
						} 
					
						if (graphvizTime % refresh_interval != 0)
						{
							std::system("CLS");
							std::cout << "Please enter a time that is a multiple of" << refresh_interval << "\n";
						} 

					} while (graphvizTime > run_time || graphvizTime < 0 || graphvizTime % refresh_interval != 0);
				}
				else if (graphviz == "EXIT")
				{
					myinfile.close();
					return 0;
				}

			}
			else
			{
				graphviz = "YES";
				while (graphvizTime <= run_time)
				{
					if (graphvizTime + refresh_interval <= run_time)
						graphvizTime += refresh_interval;
					if (graphvizTime + refresh_interval > run_time)
						break;
				}
			}

			graphvizTime -= 1;	//account for start index at 1


			std::system("CLS");

			std::cout << infilename << "\nNumber of Nodes: " << num_nodes << " Refresh Interval: " << refresh_interval << " Transmission Range: " << transmission_range << "\n\n";

			//more variables to read in node locations

			double* reached_destination = new double[num_nodes];
			std::fill_n(reached_destination,num_nodes,0);

			double* links_array = new double[num_nodes*num_nodes];
			std::fill_n(links_array,num_nodes*num_nodes,0);

			int* link_changes = new int[num_nodes];
			std::fill_n(link_changes,num_nodes,0);

			int total_link_changes = 0;

			double* x = new double[num_nodes];
			std::fill_n(x,num_nodes,0);

			double* y = new double[num_nodes];
			std::fill_n(y,num_nodes,0);

			int * index = new int[num_nodes];
			std::fill_n(index,num_nodes,0);

			int* start_index = new int[num_nodes];
			std::fill_n(start_index,num_nodes,0);

			int run_time_int = int(run_time);

			int* location_node = new int[run_time_int*num_nodes*2+1];
			std::fill_n(location_node,run_time_int*num_nodes*2+1,0);

			int* location_time = new int[run_time_int*num_nodes*2];
			std::fill_n(location_time,run_time_int*num_nodes*2,0);

			double* location_x = new double[run_time_int*num_nodes*2];
			std::fill_n(location_x,run_time_int*num_nodes*2,0);

			double* location_y = new double[run_time_int*num_nodes*2];
			std::fill_n(location_y,run_time_int*num_nodes*2,0);

			//int* animation_array = new int[(run_time_int+1)*num_nodes*2];
			//std::fill_n(animation_array,(run_time_int+1)*num_nodes*2,0);

			double* instability_value = new double[num_nodes*10];
			std::fill_n(instability_value,num_nodes*10,0);

			int* temp_link_changes = new int[num_nodes];
			std::fill_n(temp_link_changes,num_nodes,0);

			double* link_change_sum = new double[num_nodes];
			std::fill_n(link_change_sum,num_nodes,0);

			int* neighbour_array = new int[num_nodes];
			std::fill_n(neighbour_array,num_nodes,0);

			double* cluster_coefficient = new double[num_nodes];
			std::fill_n(cluster_coefficient,num_nodes,0);

			int* previous_row = new int[num_nodes];
			std::fill_n(previous_row,num_nodes,0);

			int* previous_col = new int[num_nodes];
			std::fill_n(previous_col,num_nodes,0);

			// the links array can be re-written as links_array[i*num_nodes+j] = links_array[i][j]

			//the entire array is set to 0 (ie. no connections)

			for (int i = 0; i < num_nodes; i++) 
			{

				for (int j = 0; j < num_nodes; j++) 
				{

					links_array[i*(num_nodes - 1)+j] = 0;

				} // end for

			} // end for 


			if (myinfile.is_open()) 
			{

				std::cout << "Beginning to read file...\n";

				int node_number = 0;

				std::string line;
		
				//index 0 is read in here so that location_node[i-1] does not have to be calculated.

				//basically location_node[-1] doesn't exist so this would crash a couple of if statements below

				 myinfile >> location_node[0] >> location_time[0] >> location_x[0] >> location_y[0];

				 getline(myinfile,line);

				//reads in the node #, time and location for each node untill run time	

				for (int i = 1; i <= run_time*num_nodes; i++) 
				{

					getline(myinfile,line);

					if (line.find("Node") == std::string::npos)
					{
						location_node[i] = convertToInterger(line.substr(0,line.find(" ")));
						line.erase(0,line.find(" ")+1);
						location_time[i] = convertToInterger(line.substr(0,line.find(" ")));
						line.erase(0,line.find(" ")+1);
						location_x[i] = convertToDouble(line.substr(0,line.find(" ")));
						line.erase(0,line.find(" ")+1);
						location_y[i] = convertToDouble(line);

					}
					else	//if end of node locations occurs before run time
					{
						if (node_number != num_nodes - 1)		//if not last node
						{
							reached_destination[node_number] = location_time[i-1];
							start_index[node_number+1] = i;//changed

							getline(myinfile,line);

							node_number++;

							getline(myinfile,line);
							location_node[i] = convertToInterger(line.substr(0,line.find(" ")));
							line.erase(0,line.find(" ")+1);
							location_time[i] = convertToInterger(line.substr(0,line.find(" ")));
							line.erase(0,line.find(" ")+1);
							location_x[i] = convertToDouble(line.substr(0,line.find(" ")));
							line.erase(0,line.find(" ")+1);
							location_y[i] = convertToDouble(line);

						}
						else		//if last node
						{
							end_index = i-1;// changed i or i+1 or i-1 ? why not 208?

							reached_destination[node_number] = location_time[i-1];

							myinfile.clear();

							myinfile.ignore(1000, '\n');

							myinfile >> read_line;

							break;
						}

					}//end if

					if (location_time[i] == run_time && node_number != num_nodes - 1)		//if run to end of run time go to next node skipping all other locations
					{

						reached_destination[node_number] = location_time[i-1];
						start_index[node_number+1] = i+1;//changed

						myinfile.ignore(std::numeric_limits<std::streamsize>::max(),'N');
						myinfile.ignore(256,'\n');
						myinfile.ignore(256,'\n');

						node_number++;

					}
					else if (location_time[i] == run_time && node_number == num_nodes - 1)		//if reached last node
					{

						end_index = i;

						reached_destination[node_number] = location_time[i-1];

						myinfile.clear();

						myinfile.ignore(1000, '\n');

						myinfile >> read_line;

						break;

					}

					if (myinfile.eof() == 1)
					{
						std::cout << "Reached end of file\n";
						break;
					}

				} //end for

				
				myinfile.close();

			} 
			else 
			{

				//error message if file cannot be opened

				std::cout << "Unable to open file: " << infilename << std::endl ;

				std::system("PAUSE"); 
				return 1;

			} //end if 


			//----------------------------------------------------------LOCATION READ----------------------------------------------------------//

			//---------------------------------------------------------------------------------------------------------------------------------//

			//creates files for every statistic using automatic filenames based on the input filename
			//If you wish to create a new file output, start here

			std::ofstream average_neighbours;

			std::ofstream most_neighbours;

			std::ofstream average_path_length;

			std::ofstream diameter;

			std::ofstream components;

			std::ofstream node_centrality;

			std::ofstream links;

			std::ofstream coverage;

			std::ofstream surface_graph;

			//std::ofstream animation;

			std::ofstream instability;

			std::ofstream average_instability;

			std::ofstream clustering;

			std::ofstream average_clustering;

			std::ofstream predictability;



			std::string average_neighbours_filename;

			std::string most_neighbours_filename;

			std::string average_path_length_filename;

			std::string diameter_filename;

			std::string components_filename;

			std::string node_centrality_filename;

			std::string links_filename;

			std::string coverage_filename;

			std::string surface_graph_filename;

			//std::string animation_filename;

			std::string instability_filename;

			std::string average_instability_filename;

			std::string clustering_filename;

			std::string average_clustering_filename;

			std::string predictability_filename;

			std::string graphViz_filename;


			average_neighbours_filename = infilename;


			txt_index = average_neighbours_filename.find(" output\\") + 8;


			average_neighbours_filename.erase(txt_index);


			most_neighbours_filename = average_neighbours_filename;

			average_path_length_filename = average_neighbours_filename;

			diameter_filename = average_neighbours_filename;

			components_filename = average_neighbours_filename;

			node_centrality_filename = average_neighbours_filename;

			links_filename = average_neighbours_filename;

			coverage_filename = average_neighbours_filename;

			surface_graph_filename = average_neighbours_filename;

			//animation_filename = average_neighbours_filename;

			instability_filename = average_neighbours_filename;

			average_instability_filename = average_neighbours_filename;

			clustering_filename = average_neighbours_filename;

			average_clustering_filename = average_neighbours_filename;

			predictability_filename = average_neighbours_filename;

			graphViz_filename.append(average_neighbours_filename);


			average_neighbours_filename.append("\\average_neighbours.txt");

			most_neighbours_filename.append("\\most_neighbours.txt");

			average_path_length_filename.append("\\average_path_length.txt");

			diameter_filename.append("\\diameter.txt");

			components_filename.append("\\components.txt");

			node_centrality_filename.append("\\node_centrality.txt");

			links_filename.append("\\links.txt");

			coverage_filename.append("\\coverage.txt");

			surface_graph_filename.append("\\surface_graph.txt");

			//animation_filename.append("\\animation.txt");

			instability_filename.append("\\neighbourhood_instability.txt");

			average_instability_filename.append("\\average_instability.txt");

			clustering_filename.append("\\clustering_coefficient.txt");

			average_clustering_filename.append("\\average_clustering.txt");

			predictability_filename.append("\\predictability.txt");

			if (graphviz == "YES")
			{

				std::ostringstream convert;

				convert << graphvizTime+1;

				std::string graphViz_filename_time = convert.str();

				graphViz_filename.append("\\graphViz_");

				graphViz_filename.append(graphViz_filename_time);

				graphViz_filename.append(".dot");

			}


			average_neighbours.open(average_neighbours_filename);

			most_neighbours.open(most_neighbours_filename);

			average_path_length.open(average_path_length_filename);

			diameter.open(diameter_filename);

			components.open(components_filename);

			node_centrality.open(node_centrality_filename);

			links.open(links_filename);

			coverage.open(coverage_filename);

			surface_graph.open(surface_graph_filename);

			//animation.open(animation_filename);

			instability.open(instability_filename);

			average_instability.open(average_instability_filename);

			clustering.open(clustering_filename);

			average_clustering.open(average_clustering_filename);

			predictability.open(predictability_filename);


			int elapsed_time = 0;

			int previous_time = -1;	///0

			int added_links = 0;

			int broken_links = 0;

			int total_added_links = 0;

			int total_broken_links = 0;

			int one_component = 0;
	
			//Coverage and predictability variables and arrays
			const int number_of_segments = 100; //Dimensions of coverage grid
			const int predictability_segments = 50;  //Dimensions of predictability main grid (first two dimensions)
			const int surrounding_grid = 11; //Dimensions of predictability sub-grid (last two dimensions)
			unsigned short coverage_grid[number_of_segments][number_of_segments];
			double different_states[predictability_segments][predictability_segments]; //Stores number of times a node has moved from the location in the grid
			unsigned short predictability_grid[predictability_segments][predictability_segments][surrounding_grid][surrounding_grid];

			for (int row = 0; row < number_of_segments; row++) {     //Initialize the coverage array
				for (int col = 0; col < number_of_segments; col++) {
				   coverage_grid[row][col] = 0;
				}
			}
			
			for (int g = 0; g < predictability_segments; g++) {     //Initialize the predictability arrays
				for (int h = 0; h < predictability_segments; h++) {
					different_states[g][h] = 0;
					for (int i = 0; i < surrounding_grid; i++) {
						for (int j = 0; j < surrounding_grid; j++) {
							predictability_grid[g][h][i][j] = 0;
						}
					}
				}
			}
			int total_coverage = 0;

			int xfloor = 0;
			int	yfloor = 0;

			int predictability_xfloor = 0;
			int	predictability_yfloor = 0; 

			double x_segment = x_axis/number_of_segments;  //Set division of area for coverage
			double y_segment = y_axis/number_of_segments;

			double x_predictability = x_axis/predictability_segments;  //Set division of area for predictability
			double y_predictability = y_axis/predictability_segments;

			double coverage_percent = 0; //Tracks percent of total area that has been covered by the nodes

			double same_state = 0; //Stores value representing the probability of a node remaining in the same state
			
			double current_state = 0; //Stores probability of node moving to another specific state

			double total_predictability = 0; //Tracks total sum of predictability values

			//Neighbourhood Instability and Clustering Coefficient variables

			double nodeneighbours = 0.0;

			int counter = 0;

			double instability_sum = 0;

			bool begin_output = false;

			double network_instability = 0;

			double average_instability_value = 0;

			int cluster_links = 0;

			double network_clustering = 0;

			double average_clustering_value = 0;


			index[0] = 0;

			for (int c = 1; c < num_nodes; c++) {
				index[c] = start_index[c];
			} //end for 	


			for (elapsed_time = 0; elapsed_time <= run_time; elapsed_time ++) 
			{
				
				if (elapsed_time == std::floor(time_percent*run_time))
				{
					std::system("CLS");
					std::cout << infilename << "\nNumber of Nodes: " << num_nodes << " Refresh Interval: " << refresh_interval << " Transmission Range: " << transmission_range << "\n";
					std::cout << "Creating statistics...\n\n";
					std::cout << elapsed_time << "/" << run_time << " " << time_percent*100 << "% completed" <<"\n";

					time_percent += 0.05;
				}
				
				int b = 0;

				added_links = 0;

				broken_links = 0;

				//every time the refresh interval is reached, the following is executed

				for (int c = 0; c < num_nodes; c++) 
				{

					//displays the node and the index it is at
				
					//if it is the last node then the index is set to the sum of all the reached destination array

					//ie. reached_destination[0] is the index at which node 0 has reached its destination

					if (index[c] >= end_index-1 && c == num_nodes-1)// not -1?
					{

						index[c] = end_index;


						//runs if the index is greater than the last index (redundant?)

						}
						else if (index[c] >= end_index) 
						{

							index[c] = start_index[c+1] - 1;


						//if the index is greater than the start index of the next index, then the node has reached its destination

						} 
						else if (index[c] >=  start_index[c+1] &&  c != num_nodes-1)
						{

							index[c] = start_index[c+1] - 1;

						} //end if 
						
	

						//location is set to the location at that index

						x[c] = location_x[index[c]];

						y[c] = location_y[index[c]];

						//goes to the next index

						index[c] += 1;

					} //end for


					


		//----------------------------------------------------------Generate Statistics----------------------------------------------------------//

		//---------------------------------------------------------(from Hassein Saeed)----------------------------------------------------------//

				if (elapsed_time - previous_time  >= refresh_interval) 
				{

					previous_time = elapsed_time;

					//Create a typedef for the Graph, Vertex and Edge type

					typedef adjacency_list<vecS, vecS, undirectedS, no_property, property<edge_weight_t, double>> Graph;

					typedef graph_traits<Graph>::vertex_descriptor Vertex;

					typedef graph_traits <Graph>::vertex_descriptor vertex_descriptor;

					typedef std::pair<int, int> Edge;

		
					int k = 0, m = 0,  count2 = 0, maxcount = 0, maxcountarray[100], maxdist = 0, mostnodeneigbors[100], minhops = 1;

					double count1 = 0, l = 0, sum = 0, count3 = 0, sum2 = 0, averagelength = 0, sumofhops = 0, maxconnected = 0, maxarray[100], maxconnectedarray[100];

					for (int i = 0;i < 100;i++)
					{
						maxcountarray[i] = 0;
						mostnodeneigbors[i] = 0;
						maxarray[i] = 0;
						maxconnectedarray[i] = 0;
					}
	

					//With every node's coordinates known, we must find how many edges this network will have 

					//(and therefore know if the two nodes are connected)

					for(int n = 0; n < num_nodes-1; n++) //num_nodes -1 because n at num_nodes-1 then p = num_nodes and nothing would happen
					{

						for (int p = n+1; p < num_nodes; p++) 
						{

							//Use pythagorean theorum to find the edge distance (Change in x^2 + Change in y^2 = Hypothenuse^2)

							l = sqr(x[p] - x[n]) + sqr(y[p]-y[n]);

							l = pow(l, 0.5);

							//If the edge distance is less than transmission range, than the nodes are connected

							//If this is true, store how many edges are needed into variable m so that an array can be made 

							if (l <= transmission_range) 
							{
								
								m++;

								if (links_array[n*num_nodes+p] == 0) 
								{

									links_array[n*num_nodes+p] = 1;

									links_array[p*num_nodes+n] = 1;

									if (elapsed_time != 0)
									{
										link_changes[n] +=1;
										link_changes[p] +=1;
										temp_link_changes[n] +=1;
										temp_link_changes[p] +=1;
										total_link_changes += 2;
									}

									added_links += 1;

									total_added_links += 1;

								} //end if 
								
							} 
							else if (links_array[n*num_nodes+p] == 1) 
							{												//outside transmission range

								links_array[n*num_nodes+p] = 0;

								links_array[p*num_nodes+n] = 0;

								if (elapsed_time != 0)
								{
									link_changes[n] +=1;
									link_changes[p] +=1;
									temp_link_changes[n] +=1;
									temp_link_changes[p] +=1;
									total_link_changes += 2;
								}

								broken_links += 1;

								total_broken_links += 1;

							}	//end if

						}	//end for

					}	//end for

					
					//Neighbourhood instability and clustering coefficient:
					//n indicates primary node, p is the linked neighbour
					for (int n = 0; n < num_nodes; n++) {  //Cycle through each node, determine number of neighbours
						for (int p = 0; p < num_nodes; p++) {
							if (links_array[n*num_nodes+p] == 1) {
								nodeneighbours += 1;
								neighbour_array[int(nodeneighbours)-1] = p;
								//Use nodeneighbours-1 so that the identity of neighbour 1 goes in neighbour_array[0], and can be easily scrolled through
							}
						}
						//Clustering code
						for (int q = 0; q < (nodeneighbours-1); q++) {  //Cycle through the nodes to determine links
							for (int r = q+1; r < nodeneighbours; r++) { //Do not repeat combinations
								if (links_array[neighbour_array[q]*num_nodes+neighbour_array[r]] == 1) { 
									cluster_links += 1; //If one neighbour is linked to another, increment variable
								}
							}
						}
						if (nodeneighbours != 0) {  //Ensure that there is no division by 0
							instability_value[n*10+counter] = temp_link_changes[n]/nodeneighbours; //Obtain instability of node
							cluster_coefficient[n] = cluster_links/nodeneighbours; //Obtain clustering coefficient of node
						}
						else {
							instability_value[n*10+counter] = 0;
							cluster_coefficient[n] = 0;
						}
						cluster_links = 0; //Reset values for next node
						temp_link_changes[n] = 0; 
						nodeneighbours = 0;
					}
					counter += 1; //Counter cycles through the instability value array, ensuring it always contains the last 10 seconds
					if (counter == 10) {
						counter = 0;
						begin_output = true; //Start outputting the results once one cycle has completed.
					}

					// edge_array[i][j] is rewritten as edge_array[i*num_nodes+j];

					//Now create the edges and their weights

					double* weights = new double[m];
					std::fill_n(weights,m,0);

					Edge* edge_array = new Edge[max_amount];

					int o = 0;

					for(int n = 0; n < num_nodes; n++) 
					{
						//Coverage code
						//For each node, determine the corresponding part of the coverage grid and increase it by 1
						xfloor = x[n]/x_segment; //xfloor is an integer and truncates the division, so x[n] values between 0 and (x_segment) round to 0.
						yfloor = y[n]/y_segment;
						coverage_grid[yfloor][xfloor] += 1; //Remember, coverage_grid is [row][col], with [row] as y and [col] as x.
						//The following if statement is to check for any errors causing irrational results. It will list them in the surface_graph test output.
						if (coverage_grid[yfloor][xfloor] < (-10000) || coverage_grid[yfloor][xfloor] > 10000) {
							surface_graph << "Error: Coords are " << yfloor << "," << xfloor << ". Node is " << n << ". Time is " << elapsed_time << ". Value is " << coverage_grid[yfloor][xfloor] << ". \n";
						}
						

						//Animation code stores coordinates at each point in time for each node.
						//animation_array[(elapsed_time*num_nodes*2)+(n*2)] = x[n];
						//animation_array[(elapsed_time*num_nodes*2)+(n*2)+1] = y[n];

						//Predictability code
						
						predictability_xfloor = x[n]/x_predictability;
						predictability_yfloor = y[n]/y_predictability;
						//predictability_xfloor and _yfloor have determined the current grid location, previous_col and previous_row have the previous location
						if (elapsed_time > 0) { //Skip the first cycle in order to have previous_row and previous_col values
							different_states[previous_row[n]][previous_col[n]] += 1;
							//Add 1 to the appropriate part of the surrounding grid
							predictability_grid[previous_row[n]][previous_col[n]][(predictability_yfloor-previous_row[n]+(surrounding_grid/2+1))][(predictability_xfloor-previous_col[n]+(surrounding_grid/2+1))] += 1;
						}
						previous_col[n] = predictability_xfloor; //Set the previous values to the current ones for the next cycle
						previous_row[n] = predictability_yfloor;

						for (int p = n+1; p < num_nodes; p++) 
						{

							//Use pythagorean theorem to find the edge distance (Change in x^2 + Change in y^2 = Hypothenuse^2)

							l = sqr(x[p] - x[n]) + sqr(y[p]-y[n]);

							l = pow(l, 0.5);

							//If the edge distance is less than transmission range, than the nodes are connected

							//If this is true, create an edge between the nodes and store the magnitude of the distance as the edge weight 

							if (l <= transmission_range) 
							{
					
								if (o >= m) std::cout << "array out of bounds\n" << "m: " << m << " o: " << o << "\n";

								edge_array[o] = Edge(n,p);


								if (edge_array[o].first == 0 && edge_array[o].second == 0) std::cout << "(0,0) at " << o << "\n";

								o++;

								edge_array[o] = Edge(0,1);

								weights[k] = l;

								k++;

							}	//end if

						}	//end for

					}	//end for	
					
					//Determine the percentage of the coverage grid that has been occupied
					//The correct X-axis and Y-axis values must be entered when running Opnet files, otherwise the grid does not match the area
					for (int row = 0; row < number_of_segments; row++) {
						for (int col = 0; col < number_of_segments; col++) {
							if (coverage_grid[row][col] > 0) {
								total_coverage += 1;
							}
						}
					}
					//Divide the number of cells that have been occupied by the total cells, multiple by 100 to get %
					coverage_percent = total_coverage/(sqr(number_of_segments))*100; 
					//Reset total_coverage for next cycle
					total_coverage = 0;
					



					const int num_arcs = sizeof(*edge_array)/sizeof(edge_array[0]);

	
					// declare a graph object, adding the edges and edge properties

					#if defined(BOOST_MSVC) && BOOST_MSVC <= 1300

					// VC++ can't handle the iterator constructor

					Graph G(num_nodes);

					property_map<Graph, edge_weight_t>::type weightmap = get(edge_weight, G);

					for (std::size_t j = 0; j < num_edges; ++j) 
					{

						graph_traits<Graph>::edge_descriptor t; bool inserted;

						boost::tie(t, inserted) = add_edge(edge_array[j].first, edge_array[j].second, G);

						weightmap[t] = weights[j];

					}

					#else
				
						Graph G(edge_array, edge_array + o, weights, num_nodes);

					#endif


					//Removing edges 0-0 will remove all the blank edges of the array and will only keep the existing ones

					boost::remove_edge(0, 0, G);


					//Create a new array of edge weights of all 1s and make a new graph with this

					//This will help us calculate hops

					int* weights2 = new int[m];

					for (int f = 0; f < m; f++) 
					{
						weights2[f] = 1;
					}


					//New graph with edge weights of only 1, purely just for find number of hops

					Graph gt(edge_array, edge_array + o, weights2, num_nodes);

					boost::remove_edge (0,0,gt);


					//Use propety map to create a vertex index and an edge weight

					boost::property_map<Graph, vertex_index_t>::type

					vertex_id = get(vertex_index,G);

					boost::property_map<Graph, edge_weight_t>::type

					weightmap = get(edge_weight,G);


					typedef graph_traits<Graph>::vertex_iterator vertex_iter;

					std::pair<vertex_iter, vertex_iter> vp;


					graph_traits<Graph>::edge_iterator ei, ei_end;

	
					k = 0;

	
					//Make the program outout each Node and ever Node that is connected to it

					//While doing this, take note of the average amount of neighbors( connected nodes)

					//each node has and which node(s) has the most neighbors 

					for (vp = vertices(G); vp.first != vp.second; ++vp.first) 
					{

						boost::property_map<Graph, vertex_index_t>::type

						vertex_id = get(vertex_index, G);

						count1++;


						// For each node get the outgoing edges, but only output the target node and increase count2 by 1 for each

						boost::graph_traits<Graph>::out_edge_iterator out_i, out_end;

						boost::graph_traits<Graph>::edge_descriptor e;

						for (boost::tie(out_i, out_end) = out_edges(*vp.first, G);out_i != out_end; ++out_i)
						{

							e = *out_i;

							Vertex targ = target(e, G);

							count2++;

						}	//end for


						//If this node has more or same amount of neighbors than the previous highest node,

						// save it's value in the maxcountarray and it's name in the mostnodeneighbors array 

						if (count2 >= maxcount) 
						{

							maxcount = count2;

							maxcountarray[k] = maxcount;

							mostnodeneigbors[k] = *vp.first;

							k++;

						}	//end if


						//Add up all the count2s and store them in the variable sum

						sum+= count2;

						count2 = 0;

					}	//end for


					//Calculcate Shortest Paths

					std::vector<vertex_descriptor> P(num_vertices(G));

					std::vector<int> d(num_vertices(G));

   
					for (int p=0; p < num_nodes; p++) 
					{

						vertex_descriptor s = vertex(p, G);


						#if defined(BOOST_MSVC) && BOOST_MSVC <= 1300

							// VC++ has trouble with the named parameters mechanism

							property_map<graph_t, vertex_index_t>::type indexmap = get(vertex_index, g);

							dijkstra_shortest_paths(g, s, &p[0], &d[0], weightmap, indexmap, 

							std::less<int>(), closed_plus<int>(), 

							(std::numeric_limits<int>::max)(), 0,

							default_dijkstra_visitor());

						#else

							dijkstra_shortest_paths(G, s, predecessor_map(&P[0]).distance_map(&d[0]));

						#endif

	
						graph_traits <Graph>::vertex_iterator vi, vend;

						for (boost::tie(vi, vend) = vertices(G); vi != vend; ++vi) 
						{

							if (d[*vi] >= maxdist && d[*vi] <= num_nodes*transmission_range) 
							{
								maxdist = d[*vi];
							}

							if (d[*vi] <= num_nodes*transmission_range) 
							{

								sum2+= d[*vi];	

								count3++;

							} //end if 

						}	//end for

					}	//end for


					//Calculcate Shortest Paths for the graph with edge weights all = to 1

					//This will help us calculate the distance in of hops (hops = distance)

					std::vector<vertex_descriptor> PM(num_vertices(gt));

					std::vector<int> D(num_vertices(gt));


					for (int p=0; p < num_nodes; p++) 
					{

						vertex_descriptor s = vertex(p, gt);

						#if defined(BOOST_MSVC) && BOOST_MSVC <= 1300

							// VC++ has trouble with the named parameters mechanism

							property_map<graph_t, vertex_index_t>::type indexmap = get(vertex_index, g);

							dijkstra_shortest_paths(g, s, &p[0], &d[0], weightmap, indexmap, 

							std::less<int>(), closed_plus<int>(), 

							(std::numeric_limits<int>::max)(), 0,

							default_dijkstra_visitor());

						#else

							dijkstra_shortest_paths(gt, s, predecessor_map(&PM[0]).distance_map(&D[0]));

						#endif


						graph_traits <Graph>::vertex_iterator vi, vend;

						for (boost::tie(vi, vend) = vertices(G); vi != vend; ++vi) 
						{

							if (D[*vi] >= minhops && D[*vi] <= num_nodes) 
							{
								minhops = D[*vi];
							}

							if (D[*vi] <= num_nodes) 
							{
								sumofhops = sumofhops + D[*vi];
							} //end if 

						}	//end for

					}	//end for


					//Calculate the connected components of the network

					std::vector<int> component(num_vertices(G));

					int num = connected_components(G, &component[0]);

					if (num == 1) {
						one_component += 1; //Keep track of how often there is only one component
					}

					//Find the Betweeness Centrality of the network

					typedef graph_traits<Graph>::vertex_iterator Vertex_Iter; 

					Vertex_Iter vertex_iterator_begin, vertex_iterator_end; 

					std::vector<double> centrality(num_nodes); 
        

					boost::brandes_betweenness_centrality(G, 

					centrality_map(make_iterator_property_map(centrality.begin(), 

					get(vertex_index, G),double())).vertex_index_map(get(vertex_index, 

					G))); 

    
					o = 0;


					for ( tie(vertex_iterator_begin, vertex_iterator_end) = 

					vertices(G); vertex_iterator_begin != 

					vertex_iterator_end; ++vertex_iterator_begin) 
					{	

						if (centrality[*vertex_iterator_begin] >= maxconnected) 
						{

							maxconnected = centrality[*vertex_iterator_begin];

							maxarray[o] = centrality[*vertex_iterator_begin];

							maxconnectedarray[o] = *vertex_iterator_begin;

							o++;

						}	//end if

					}	//end for	


		//----------------------------------------------------------Output the Statistics----------------------------------------------------------//

		//-----------------------------------------------------------------------------------------------------------------------------------------//


					int* mostneighbours = new int[100];
					std::fill_n(mostneighbours,100,0);

					if (k >= 1)
					{
						mostneighbours[0] = mostnodeneigbors[k - 1];
					}
					else
						std::cout << "!!";

					int z = 1;

					for (int n = k - 2; n >= 0; n--) 
					{

						if (maxcountarray[n] == maxcountarray[n+1]) 
						{

							int s = mostnodeneigbors[n];

							mostneighbours[z] = mostnodeneigbors[n];

							z++;

						}	//end if

					}	//end for



					count3 = (num_nodes-1)*(num_nodes);

					averagelength = sum2/count3;
	

					double* centralnodes = new double[num_nodes];
					std::fill_n(centralnodes,num_nodes,0.0);

					if (o >= 1)
					{
						centralnodes[0] = maxconnectedarray[o-1];
					}
					else
						std::cout << "!!";

					int w = 1;

					for (int n = o - 2; n >= 0; n--) 
					{

						if (maxarray[n] == maxarray[n+1]) 
						{
							centralnodes[w] = maxconnectedarray[n];
							w++;
						} 
						else 
						{
							break;
						}	//end if

					}	//end for



		//-------------------------------------------------------Write the Statistics to File------------------------------------------------------

		//-----------------------------------------------------------------------------------------------------------------------------------------


					average_neighbours << elapsed_time << " " << sum/count1 << std::endl;

					most_neighbours << elapsed_time; 

					for (int i = 0; i < z ; i++) 
					{

						most_neighbours << " " << mostneighbours[i];

					} //end for

					if (k >= 1)
					{
						most_neighbours << " (" << maxcountarray[k-1] << ")" << std::endl;
					}
					else
						std::cout << "!!";

					average_path_length << elapsed_time << " " << sumofhops/count3  << " " <<  averagelength << std::endl;

					diameter << elapsed_time << " " <<  minhops << " " << maxdist << std::endl;

					components << elapsed_time << " " << num << std::endl;


					node_centrality << elapsed_time;


					for (int i = 0; i < w; i++) 
					{

						node_centrality << " " << centralnodes[i];

					} //end for


					node_centrality << std::endl;	


					links << elapsed_time << " " << added_links << " " << broken_links << std::endl;

					if (elapsed_time == 0)
					{
						total_added_links = total_added_links - added_links;		//accounts for all the link added at start time
					}

					coverage << elapsed_time << " " << coverage_percent << std::endl;

					if (begin_output) {
						instability.precision(3); //Precision prevents outputs from being too long, as this outputs values for each node.
						average_instability.precision(3);
						instability << (elapsed_time-9) << " " << elapsed_time;
						for (int n = 0; n < num_nodes; n++) {
							for (int i = 0; i < 10; i++) {
								instability_sum += instability_value[n*10+i]; //Total up the instability values in the last-10-seconds array
							}
							instability << " " << instability_sum;
							network_instability += instability_sum; //Stores the instability of all nodes at each point in time, outputs the average.
							instability_sum = 0; //Reset the total each time
						}
						instability << std::endl;
						average_instability_value += network_instability; //Stores all instability values total, averages at the end.
						network_instability = network_instability/num_nodes;
						average_instability << (elapsed_time-9) << " " << elapsed_time << " " << network_instability << std::endl;
						network_instability = 0; //Reset for next cycle
					}

					clustering << elapsed_time;
					clustering.precision(3); //Again, precision prevents outputs from being too long
					average_clustering.precision(3);
					for (int i = 0; i < num_nodes; i++) {
						clustering << " " << cluster_coefficient[i];
						network_clustering += cluster_coefficient[i]; //Stores the clustering of all nodes at each point in time, outputs the average.
					}
					clustering << std::endl;
					average_clustering_value += network_clustering; //Stores all clustering values total, averages at the end.
					network_clustering = network_clustering/num_nodes;
					average_clustering << elapsed_time << " " << network_clustering << std::endl;
					network_clustering = 0; //Reset for next cycle

					bool visualization = true;


					//output the GraphViz code
					if (graphviz == "YES" && graphvizTime == elapsed_time) 
					{

						graphviz = "NO";

						std::ofstream graphVizoutfile (graphViz_filename);
						graphVizoutfile << "Graph G {" << std::endl;

						for (k = 0; k < num_nodes; k++) 
						{
							graphVizoutfile << k << " [pos=" << " \"" << x[k] << "," << y[k] << "!\"];" << std::endl;
						}

						for (boost::tie(ei, ei_end) = edges(G); ei != ei_end; ++ei) 
						{
							graphVizoutfile << source(*ei, G) << "--" << target(*ei,G) <<";" << std::endl;
						}

						graphVizoutfile << "}";

						graphVizoutfile.close();

					}


					//delete pointers

					delete [] centralnodes;

					delete [] mostneighbours;

					delete [] weights;

					delete [] weights2;

					delete [] edge_array;

				} //end if 

				//system("PAUSE");		//debugging location set

			} //end for


		//----------------------------------------------------------End of Program & Variable Delete---------------------------------------------//
			
		//---------------------------------------------------------------------------------------------------------------------------------------//

			//displays total links added and broken and averages over runtime

			//writes them to file as well

			links << "Total added links: " << total_added_links << "\nTotal broken links: " << total_broken_links << "\n";

			links << "Average links added per sec: " << total_added_links/(double(run_time) - 1) << "\nAverage links broken per sec: " << total_broken_links/double(run_time ) << "\n";

			links << "\nLink Changes: " << total_link_changes << "\n";

			links << "\nNode\t\t| Link Changes\n";
			for (int d = 0;d < num_nodes;d++)
			{
				links << d << "\t\t| " << link_changes[d] << "\n";
			}

			components << "Percent of time with one component: " << (one_component/(run_time+1)*100) << "%.\n";

			//Coverage Code: Rough visualization
			//Display the area dimensions, the segment dimensions, and the final status of the coverage grid
			coverage << "\n" << "X-Axis: " << x_axis << " Y-Axis: " << y_axis << " X segment: " << x_segment << " Y segment: " << y_segment << "\n";
			coverage << "Final Coverage Array: \n";
			for (int row = 0; row < number_of_segments; row++) {
				for (int col = 0; col < number_of_segments; col++) {
					coverage << coverage_grid[row][col] << " ";
				}
				coverage << "\n";
			}
			coverage << "\n";

			//Seperate output giving the code to create an array in MATLAB, allowing for surface graphs
			surface_graph << "Matlab matrix code: [";
			for (int row = 0; row < number_of_segments; row++) {
				for (int col = 0; col < number_of_segments; col++) {
					surface_graph << coverage_grid[row][col] << " ";
				}
				surface_graph << ";";
			}
			surface_graph << "];\n";

			//Animation output, in form of MATLAB code
			//for (int time = 1; time < (run_time+1); time++) {
			//	animation << "A(:,:," << time << ") = [";
			//	for (int row = 0; row < num_nodes; row++) { 
			//		animation << animation_array[(time*num_nodes*2)+(row*2)] << " "<< animation_array[(time*num_nodes*2)+(row*2)+1] << ";";
			//	}
			//	animation << "];\n";
			//}
			//Code auto-displays the resulting animation and records it using M(k)
			//animation << "for k = 1:" << run_time << "\n" << "scatter(A(:,1,k),A(:,2,k))\n";
			//animation << "axis([0," << x_axis << ",0," << y_axis << "])\n";
			//animation << "M(k) = getframe;\n" << "end\n";

			//Output average values for instability and clustering, for use in future statistics summaries and graphing
			average_instability_value = average_instability_value/num_nodes/(run_time-8);
			instability << "\n" <<"Average neighbourhood instability is " << average_instability_value << "\n";

			average_clustering_value = average_clustering_value/num_nodes/(run_time+1);
			clustering << "\n" <<"Average clustering coefficient is " << average_clustering_value << "\n";

			//Predictability calculations
			for (int row = 0; row < predictability_segments; row++) {
				for (int col = 0; col < predictability_segments; col++) {
					same_state = (predictability_grid[row][col][(surrounding_grid/2+1)][(surrounding_grid/2+1)]/different_states[row][col]);
					for (int i = 0; i < surrounding_grid; i++) {
						for (int j = 0; j < surrounding_grid; j++) {
							if (i != (surrounding_grid/2+1) && j != (surrounding_grid/2+1) && run_time > 0) {
								if (predictability_grid[row][col][i][j] != 0) {
									current_state = (predictability_grid[row][col][i][j]/different_states[row][col]);
									total_predictability += same_state*current_state*log(current_state);
								}
							}
						}
					}
				}
			}
			total_predictability = total_predictability/(predictability_segments*predictability_segments)*(-1);
			predictability << total_predictability << "\n";


			//close all opened text files

			average_neighbours.close();

			most_neighbours.close();

			average_path_length.close();

			diameter.close();

			components.close();

			node_centrality.close();

			links.close();

			coverage.close();

			//animation.close();

			instability.close();

			average_instability.close();

			clustering.close();

			average_clustering.close();

			predictability.close();


			//delete pointers

			delete [] reached_destination;

			delete [] x;

			delete [] y;

			delete [] links_array;

			delete [] link_changes;

			delete [] index;

			delete [] location_node;

			delete [] location_time;

			delete [] location_x;

			delete [] location_y;

			//delete [] animation_array;

			delete [] instability_value;

			delete [] temp_link_changes;

			delete [] link_change_sum;

			delete [] neighbour_array;

			delete [] cluster_coefficient;

			delete [] previous_row;

			delete [] previous_col;

			delete [] start_index;

			std::system("CLS");

			std::cout << "Statistics for " << infilename << " have successfully been created\n";

		}	//end if

	}	//end if


} while (exitmask != "EXIT" && run == false);

return 0;

}	//end DynamicStatistics()


/*
	StatisticsSummary function:
		CReates summary of statistics for a batch of files.
		locates files by user defined sequence
		outputs files in summary folder
		simple averaging of files by adding all values then dividing by amount of files
*/
int StatisticsSummary()
{

 //------------------------------------------------------------------------------VARIABLES--------------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//
   
	double matches = 0;

	int refresh_rate = 0;

	std::string path = "";
	std::string initialpath = "";

	std::string sequence = "";

	std::string filePath1 = "";
	std::string filePath2 = "";

	std::string filename1 = "";
	std::string filename2 = "";

	std::string pattern = "";

	std::string extension = "";

	double runtime = 0;
	
	bool correct;

 //------------------------------------------------------------------------------PROGRAM-----------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


	std::cout << "\n";
	std::cout << "\t***********************************************\n";
	std::cout << "\t*                                             *\n";
	std::cout << "\t*             Statistics Summary              *\n";
	std::cout << "\t*                                             *\n";
	std::cout << "\t*                                             *\n";
	std::cout << "\t*          Enter the path to the files        *\n";
	std::cout << "\t*                                             *\n";
	std::cout << "\t*        Enter EXIT to exit to main menu      *\n";
	std::cout << "\t*                                             *\n";
	std::cout << "\t***********************************************\n";
	std::cout << "\n\n";

	std::cout << "\t            >> "; getline(std::cin,path);

	exitmask = path;

	to_upper(exitmask);

	if (exitmask != "EXIT")	//if user chose not to exit
	{

		if (path[path.length()-1] != '\\')	//if end of file path doesn't have a \ add one
			path.append("\\");

		system("CLS");

		if (dirExists(path) == true)		//if path leads to valid directory
		{

			do 
			{		
  
				std::cout << "\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*             Statistics Summary              *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*            Choose the file type             *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*                   NS2                       *\n";
				std::cout << "\t*                  OPNET                      *\n";
				std::cout << "\t*                  CORE                       *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*        Enter EXIT to exit to main menu      *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\n\n";
          
				std::cout << "\t          File Type: >> "; std::getline(std::cin,filetype);
		
 
				to_upper(filetype);

				if(filetype != "NS2" && filetype != "OPNET" && filetype != "CORE" && filetype != "EXIT")
				{
					system("CLS");
					std::cout << "The file type you have entered is unrecognizable, please try again.\n" << std::endl;
				}

			}while (filetype != "NS2" && filetype != "OPNET" && filetype != "CORE" && filetype != "EXIT");


			if (filetype == "NS2")			//set valid extension for each file type
				extension = "*.txt";
			else if (filetype == "OPNET")
				extension = "*.xml";
			else if (filetype == "CORE")
				extension = "*.scen";
			else if (filetype != "EXIT")
				return 0;

			initialpath = path + extension;


			std::system("CLS");

			std::cout << "\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*             Statistics Summary              *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*   Enter the sequence you wish to match for  *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*        Enter EXIT to exit to main menu      *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\n\n";

			std::cout << "\t            >> "; getline(std::cin,sequence);

			exitmask = sequence; to_upper(exitmask);
			if (exitmask == "EXIT")
				return 0;

			pattern = "\\?*" + sequence + "\\?*";		//pattern for file with any extension and any word before and after the desired sequence

			system("CLS");


			do
			{

				correct = true;

				std::cout << "\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*             Statistics Summary              *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*    Enter the run time for the statistics    *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*        Enter EXIT to exit to main menu      *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\n\n";

				std::cout << "\t            >> "; getline(std::cin,temp);

				exitmask = temp; to_upper(exitmask);
				if (exitmask == "EXIT")
					return 0;

				system("CLS");	

				for (size_t i = 0; i < temp.length() && correct == true;i++)
				{
					if (temp[i] > '9' || temp[i] < '0')
					{
						correct = false;
						std::system("CLS");
						std::cout << "Invalid input\n";
					}
				}

				if (correct == true)
				{
					runtime = convertToDouble(temp);
				}

			}while (correct == false);


			double** components;
			double** neighbours;
			double** links;
			double** pathlengths;
			double** diameters;
			double** coverage;
			double** clustering;
			double** instability;
			double* predictability = new double[2];
			
			

			components = new double*[max_amount];
			neighbours = new double*[max_amount];
			links = new double*[max_amount];
			pathlengths = new double*[max_amount];
			diameters = new double*[max_amount];
			coverage = new double*[max_amount];
			clustering = new double*[max_amount];
			instability = new double*[max_amount];
			//predictability = new double*[max_amount];

			for (int i = 0; i < max_amount; i++)		//sets arrays to null
			{
				components[i] = new double[2];
				neighbours[i] = new double[2];
				links[i] = new double[4];
				pathlengths[i] = new double[4];
				diameters[i] = new double[4];
				coverage[i] = new double[2];
				clustering[i] = new double[2];
				instability[i] = new double[2];
				//predictability[i] = new double[2];
			}

			double* componentsSUM = new double[max_amount];
			double* neighboursSUM = new double[max_amount];
			double* linksaddedSUM = new double[max_amount];
			double* linkslostSUM = new double[max_amount];
			double* linksaddedTOTAL = new double[max_amount];
			double* linkslostTOTAL = new double[max_amount];
			double* pathlengthsSUM_hops = new double[max_amount];
			double* diametersSUM_hops = new double[max_amount];
			double* pathlengthsSUM_meters = new double[max_amount];
			double* diametersSUM_meters = new double[max_amount];
			double* coveragepercentSUM = new double[max_amount];
			double* clusteringSUM = new double[max_amount];
			double* instabilitySUM = new double[max_amount];
			//double* predictabilitySUM = new double[max_amount];
			double predictabilitySUM = 0;

			for (int i = 0; i < max_amount; i++)		//sets arrays to null
			{
				componentsSUM[i] = 0;
				neighboursSUM[i] = 0;
				linksaddedSUM[i] = 0;
				linkslostSUM[i] = 0;
				linksaddedTOTAL[i] = 0;
				linkslostTOTAL[i] = 0;
				pathlengthsSUM_hops[i] = 0;
				diametersSUM_hops[i] = 0;
				pathlengthsSUM_meters[i] = 0;
				diametersSUM_meters[i] = 0;
				coveragepercentSUM[i] = 0;
				clusteringSUM[i] = 0;
				instabilitySUM[i] = 0;
				for (int a = 0; a < 4;a++)
				{
					if (a < 2)
					{
						components[i][a] = 0;
						neighbours[i][a] = 0;
						coverage[i][a] = 0;
						clustering[i][a] = 0;
						instability[i][a] = 0;
						predictability[a] = 0;
					}
					links[i][a] = 0;
					pathlengths[i][a] = 0;
					diameters[i][a] = 0;
				}
			}
			


			std::string file_names = "";

			std::regex rxpattern (pattern);

			std::smatch match;

			WIN32_FIND_DATA FData;
			HANDLE hFind;

			hFind = FindFirstFile( initialpath.c_str(), &FData );

			if( hFind == INVALID_HANDLE_VALUE )
			{
				system("CLS");
				std::cout << "\n\nInvalid handle\n\n";
				std::system("PAUSE");
				system("CLS");
				return 1;
			}
			else    //path was found
			{

				std::cout << "Beginning to search folder...\n";

				int runtime_check = 0;

				std::ifstream infile1;
				std::ifstream infile2;

				do		//do-while loop: loops while files are still being located
				{

					filename1 = FData.cFileName;

					if (std::regex_search (filename1,match,rxpattern) == true)	//if first file name contains sequence of search
					{

						file_names.append(filename1 + "\n");

						filePath1 = path + filename1;
						matches++;
						filePath1.erase(filePath1.find("."),std::string::npos);
						filePath1.append(" output\\");

						system("CLS");
						std::cout << "Beginning to search folder...\n";
						std::cout << matches << " files found\n";

						do 
						{

							FindNextFile(hFind, &FData);
							filename2 = FData.cFileName;	

							if (std::regex_search (filename2,match,rxpattern) == true && filename2 != filename1)	//if second file name contains sequence of search
							{

								file_names.append(filename2 + "\n");

								filePath2 = path + filename2;
								matches++;
								filePath2.erase(filePath2.find("."),std::string::npos);
								filePath2.append(" output\\");

								system("CLS");
								std::cout << "Beginning to search folder...\n";
								std::cout << matches << " files found\n";


								std::string runtimefilename = filePath1 + "run_time.txt";
								infile1.open(runtimefilename);

								if (infile1.is_open())		//read in run time
								{
									infile1 >> runtime_check;
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open run time file:\n";
									std::cout << runtimefilename << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								std::string refreshfilename = filePath1 + filename1.erase(filename1.find(extension.substr(1,std::string::npos)),std::string::npos);
								refreshfilename.append("_out.txt");

								infile1.open(refreshfilename);		//reads in refresh rate (first file)

								if (infile1.is_open())
								{
									infile1 >> temp; infile1 >> temp; infile1 >> refresh_rate;
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open output file:\n";
									std::cout << refreshfilename << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								if (runtime_check < runtime * refresh_rate)
								{
									std::cout << "WARNING: Specified run time exceeds "<< filename1 << " run time\n";
									std::cout << "This will cause the statistics past this point to be calculated wrong\n";
									std::cout << runtime*refresh_rate << " < " << runtime_check << "\n";
									system("PAUSE");		
								}

								runtimefilename = filePath2 + "run_time.txt";
								infile1.open(runtimefilename);

								if (infile1.is_open())		//read in run time
								{
									infile1 >> runtime_check;
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open run time file:\n";
									std::cout << runtimefilename << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								refreshfilename = filePath2 + filename2.erase(filename2.find(extension.substr(1,std::string::npos)),std::string::npos);
								refreshfilename.append("_out.txt");

								infile1.open(refreshfilename);			//reads in refresh rate (second file)

								if (infile1.is_open())
								{
									infile1 >> temp; infile1 >> temp; infile1 >> refresh_rate;
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open output file:\n";
									std::cout << refreshfilename << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								if (runtime_check < runtime * refresh_rate)
								{
									std::cout << "WARNING: Specified run time exceeds "<< filename2 << " run time\n";
									std::cout << "This will cause the statistics past this point to be calculated wrong\n";
									std::cout << runtime*refresh_rate << " > " << runtime_check << "\n";	
								}


								filePath1.append("components.txt");		//read in components
								filePath2.append("components.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> components[i][0];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open components file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> components[i][1];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open components file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}


								filePath1.erase(filePath1.find("components.txt"),std::string::npos);
								filePath2.erase(filePath2.find("components.txt"),std::string::npos);


								filePath1.append("average_neighbours.txt");		//read in average neighbours
								filePath2.append("average_neighbours.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> neighbours[i][0];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open average neighbours file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> neighbours[i][1];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open average neighbours file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}


								filePath1.erase(filePath1.find("average_neighbours.txt"),std::string::npos);
								filePath2.erase(filePath2.find("average_neighbours.txt"),std::string::npos);


								filePath1.append("links.txt");		//read in links
								filePath2.append("links.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> links[i][0]; infile1 >> links[i][1];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open links file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> links[i][2]; infile2 >> links[i][3];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open links file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}


								filePath1.erase(filePath1.find("links.txt"),std::string::npos);
								filePath2.erase(filePath2.find("links.txt"),std::string::npos);


								filePath1.append("diameter.txt");		//read in diameters
								filePath2.append("diameter.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> diameters[i][0]; infile1 >> diameters[i][1];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open diameter file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> diameters[i][2]; infile2 >> diameters[i][3];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open diameter file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								filePath1.erase(filePath1.find("diameter.txt"),std::string::npos);
								filePath2.erase(filePath2.find("diameter.txt"),std::string::npos);


								filePath1.append("average_path_length.txt");		//read in path lengths
								filePath2.append("average_path_length.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> pathlengths[i][0]; infile1 >> pathlengths[i][1];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open path lengths file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> pathlengths[i][2]; infile2 >> pathlengths[i][3];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open path lengths file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								filePath1.erase(filePath1.find("average_path_length.txt"),std::string::npos);
								filePath2.erase(filePath2.find("average_path_length.txt"),std::string::npos);

								filePath1.append("coverage.txt");		//read in coverage
								filePath2.append("coverage.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> coverage[i][0];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open coverage file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> coverage[i][1];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open coverage file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}


								filePath1.erase(filePath1.find("coverage.txt"),std::string::npos);
								filePath2.erase(filePath2.find("coverage.txt"),std::string::npos);

								filePath1.append("average_clustering.txt");		//read in clustering
								filePath2.append("average_clustering.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile1 >> temp; infile1 >> clustering[i][0];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open clustering file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= runtime;i++)
									{
										infile2 >> temp; infile2 >> clustering[i][1];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open clustering file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}


								filePath1.erase(filePath1.find("average_clustering.txt"),std::string::npos);
								filePath2.erase(filePath2.find("average_clustering.txt"),std::string::npos);

								filePath1.append("average_instability.txt");		//read in instability
								filePath2.append("average_instability.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									for (int i = 0; i <= (runtime-9);i++)
									{
										infile1 >> temp; infile1 >> temp; infile1 >> instability[i][0];
									}
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open instability file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									for (int i = 0; i <= (runtime-9);i++)
									{
										infile2 >> temp; infile2 >> temp; infile2 >> instability[i][1];
									}
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open instability file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								filePath1.erase(filePath1.find("average_instability.txt"),std::string::npos);
								filePath2.erase(filePath2.find("average_instability.txt"),std::string::npos);

								filePath1.append("predictability.txt");		//read in predictability
								filePath2.append("predictability.txt");

								infile1.open(filePath1);

								if (infile1.is_open())
								{
									infile1 >> predictability[0];
									infile1.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open predictability file:\n";
									std::cout << filePath1 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								infile2.open(filePath2);

								if (infile2.is_open())
								{
									
									infile2 >> predictability[1];
									infile2.close();
								}
								else
								{
									system("CLS");
									std::cout << "Unable to open predictability file:\n";
									std::cout << filePath2 << "\n";
									std::system("PAUSE");
									system("CLS");
									return 1;
								}

								filePath1.erase(filePath1.find("predictability.txt"),std::string::npos);
								filePath2.erase(filePath2.find("predictability.txt"),std::string::npos);

								for (int i = 0; i <= runtime;i++)		//and all to the summary arrays
								{

									componentsSUM[i] += (components[i][0] + components[i][1])/2;
									neighboursSUM[i] += (neighbours[i][0] + neighbours[i][1])/2;
									coveragepercentSUM[i] += (coverage[i][0] + coverage[i][1])/2;
									clusteringSUM[i] += (clustering[i][0] + clustering[i][1])/2;
									instabilitySUM[i] += (instability[i][0] + instability[i][1])/2;
									linksaddedSUM[i] += (links[i][0] + links[i][2])/2;
									linkslostSUM[i] += (links[i][1] + links[i][3])/2;
									linksaddedTOTAL[i] += links[i][0] + links[i][2];
									linkslostTOTAL[i] += links[i][1] + links[i][3];
									pathlengthsSUM_hops[i] += (pathlengths[i][0] + pathlengths[i][2])/2;
									pathlengthsSUM_meters[i] += (pathlengths[i][1] + pathlengths[i][3])/2;
									diametersSUM_hops[i] += (diameters[i][0] + diameters[i][2])/2;
									diametersSUM_meters[i] += (diameters[i][1] + diameters[i][3])/2;
									

								}	//end for
								predictabilitySUM += (predictability[0] + predictability[1])/2;

								for (int i = 0; i < max_amount; i++)		//sets arrays to null
								{
									for (int a = 0; a < 4;a++)
									{
										if (a < 2)
										{
											components[i][a] = 0;
											neighbours[i][a] = 0;
											coverage[i][a] = 0;
											clustering[i][a] = 0;
											instability[i][a] = 0;
											predictability[a] = 0;
										}
										pathlengths[i][a] = 0;
										diameters[i][a] = 0;
										links[i][a] = 0;
									}
								}

							}	//end if

						}while(std::regex_search (filename2,match,rxpattern) == false && GetLastError() != ERROR_NO_MORE_FILES);

					}	//end if

				}while( FindNextFile(hFind, &FData) > 0 );
 
				if( GetLastError() != ERROR_NO_MORE_FILES )
				{
					std::cout << "\n\nERROR: did not reach end of files\n\n";
					std::cout << "Last file read " << FData.cFileName << "\n\n";
					std::system("PAUSE");
					system("CLS");
				}

				if (matches > 1)
				{

					for (int i = 0; i <= runtime;i++)		//and all to the summary arrays
					{

						componentsSUM[i] += (components[i][0] + components[i][1])/2;
						neighboursSUM[i] += (neighbours[i][0] + neighbours[i][1])/2;
						coveragepercentSUM[i] += (coverage[i][0] + coverage[i][1])/2;
						clusteringSUM[i] += (clustering[i][0] + clustering[i][1])/2;
						instabilitySUM[i] += (instability[i][0] + instability[i][1])/2;
						linksaddedSUM[i] += (links[i][0] + links[i][2])/2;
						linkslostSUM[i] += (links[i][1] + links[i][3])/2;
						linksaddedTOTAL[i] += links[i][0] + links[i][2];
						linkslostTOTAL[i] += links[i][1] + links[i][3];
						pathlengthsSUM_hops[i] += (pathlengths[i][0] + pathlengths[i][2])/2;
						pathlengthsSUM_meters[i] += (pathlengths[i][1] + pathlengths[i][3])/2;
						diametersSUM_hops[i] += (diameters[i][0] + diameters[i][2])/2;
						diametersSUM_meters[i] += (diameters[i][1] + diameters[i][3])/2;
						

					}	//end for
					predictabilitySUM += (predictability[0] + predictability[1])/2;

					matches = matches / 2;

					for (int i = 0; i <= runtime;i++)		//divide by number of pairs for average
					{
						componentsSUM[i] = componentsSUM[i] / matches;
						neighboursSUM[i] = neighboursSUM[i] / matches;
						linksaddedSUM[i] = linksaddedSUM[i] / matches;
						linkslostSUM[i] = linkslostSUM[i] / matches;
						pathlengthsSUM_hops[i] = pathlengthsSUM_hops[i] / matches;
						diametersSUM_hops[i] = diametersSUM_hops[i] / matches;
						pathlengthsSUM_meters[i] = pathlengthsSUM_meters[i] / matches;
						diametersSUM_meters[i] = diametersSUM_meters[i] / matches;
						coveragepercentSUM[i] = coveragepercentSUM[i] / matches;
						clusteringSUM[i] = clusteringSUM[i] / matches;
						instabilitySUM[i] = instabilitySUM[i] / matches;
					}
					predictabilitySUM = predictabilitySUM / matches;

							//write to output file

					system("CLS");
					std::cout << "Creating ouput files...\n";

					std::ofstream outfile;

					std::string folderpath = path + sequence +".summary";

					if (CreateDirectory(folderpath.c_str(),NULL) == 0)			//check for errors
					{
						if (ERROR_PATH_NOT_FOUND == GetLastError())	
						{
							std::cout << "\nError: path for folder creation not found.\n";
			
							folderpath = "";		//sets folder path to nothing so file will be written in same directory as input file
						}
						else if (ERROR_ALREADY_EXISTS == GetLastError())
						{
							std::cout << "\n" << folderpath << " already exists.\nOutput file will be created within the folder.\n";
						}
					}

					folderpath.append("\\");

					std::string fulloutname = folderpath + "components_summary.txt";

					double total = 0;

					outfile.open(fulloutname);			//output components summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << componentsSUM[i] << "\n";
							total += componentsSUM[i];
						}

						outfile << "\n" << "Average: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}

					fulloutname.erase(fulloutname.find("components_summary.txt"),std::string::npos);
					fulloutname.append("average_neighbours_summary.txt");


					total = 0;

					outfile.open(fulloutname);		//output neighbours summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << neighboursSUM[i] << "\n";
							total += neighboursSUM[i];
						}

						outfile << "\n" << "Average: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}




					fulloutname.erase(fulloutname.find("average_neighbours_summary.txt"),std::string::npos);
					fulloutname.append("links_summary.txt");


					total = 0;

					double total2 = 0;

					outfile.open(fulloutname);		//output links summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << linksaddedSUM[i] << " " << linkslostSUM[i] << "\n";
							if (i != 0)
							{
								total += linksaddedSUM[i];		//ignores 0 because thats when all nodes become linked
								total2 += linkslostSUM[i];
							}
						}

						outfile << "\n" << "Average added: " << total/(runtime-1) << "\n";
						outfile << "Average lost: " << total2/(runtime-1) << "\n";

						total = 0;
						total2 = 0;

						for (int i = 0;i <= runtime;i++)
						{
							if (i != 0)
							{
								total += linksaddedTOTAL[i];		//ignores 0 because thats when all nodes become linked
								total2 += linkslostTOTAL[i];
							}
						}

						outfile << "Total added: " << total << "\n";
						outfile << "Total lost: " << total2 << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}


					fulloutname.erase(fulloutname.find("links_summary.txt"),std::string::npos);
					fulloutname.append("average_path_length_summary.txt");

					total = 0;
					total2 = 0;

					outfile.open(fulloutname);		//output path length summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << pathlengthsSUM_hops[i] << " " << pathlengthsSUM_meters[i] << "\n";
							total += pathlengthsSUM_meters[i];
							total2 += pathlengthsSUM_hops[i];
						}

						outfile << "\n" << "Average hops: " << total2/runtime << "\n";
						outfile << "Average meters: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}

					fulloutname.erase(fulloutname.find("average_path_length_summary.txt"),std::string::npos);
					fulloutname.append("diameter_summary.txt");

					total = 0;
					total2 = 0;

					outfile.open(fulloutname);		//output diameter summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << diametersSUM_hops[i] << " " << diametersSUM_meters[i] << "\n";
							total += diametersSUM_meters[i];
							total2 += diametersSUM_hops[i];
						}

						outfile << "\n" << "Average hops: " << total2/runtime << "\n";
						outfile << "Average meters: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}

					fulloutname.erase(fulloutname.find("diameter_summary.txt"),std::string::npos);
					fulloutname.append("coverage_summary.txt");


					total = 0;

					outfile.open(fulloutname);		//output coverage summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << coveragepercentSUM[i] << "\n";
							total += coveragepercentSUM[i];
						}

						outfile << "\n" << "Average: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}




					fulloutname.erase(fulloutname.find("coverage_summary.txt"),std::string::npos);
					fulloutname.append("clustering_summary.txt");

					total = 0;

					outfile.open(fulloutname);		//output clustering summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= runtime;i++)
						{
							outfile << i << " " << clusteringSUM[i] << "\n";
							total += clusteringSUM[i];
						}

						outfile << "\n" << "Average: " << total/runtime << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}




					fulloutname.erase(fulloutname.find("clustering_summary.txt"),std::string::npos);
					fulloutname.append("instability_summary.txt");

					total = 0;

					outfile.open(fulloutname);		//output instability summary

					if (outfile.is_open())
					{

						for (int i = 0;i <= (runtime-9);i++)
						{
							outfile << i << " " << (i+9) << " " << instabilitySUM[i] << "\n";
							total += instabilitySUM[i];
						}

						outfile << "\n" << "Average: " << total/(runtime-9) << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}




					fulloutname.erase(fulloutname.find("instability_summary.txt"),std::string::npos);
					fulloutname.append("predictability_summary.txt");

					total = 0;
					outfile.open(fulloutname);		//output predictability summary

					if (outfile.is_open())
					{

						
						outfile << predictabilitySUM << "\n";

						outfile.close();

					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}


					fulloutname.erase(fulloutname.find("predictability_summary.txt"),std::string::npos);
					fulloutname.append("files.txt");

					outfile.open(fulloutname);		//output names of files used in summary

					if (outfile.is_open())
					{
						outfile << file_names;
						outfile.close();
					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}

					fulloutname.erase(fulloutname.find("files.txt"),std::string::npos);
					fulloutname.append("run_time.txt");

					outfile.open(fulloutname);		//output user specified run time

					if (outfile.is_open())
					{
						outfile << runtime;
						outfile.close();
					}
					else
					{
						std::cout << "Unable to create output file:\n" << fulloutname << "\n";
					}

				}//end if
				else if(matches == 0)
				{
					system("CLS");
					std::cout << "No matches for " << sequence << " found in any files of the directory:\n";
					std::cout << path;
					std::system("PAUSE");
					system("CLS");
				}
				else if (matches == 1)
				{
					system("CLS");
					std::cout << "Only 1 file was found matching " << sequence << " in the directory:\n";
					std::cout << path << "\n";
					system("PAUSE");
					system("CLS");
				}

			}//end if

			
			//delete pointers

			delete [] componentsSUM;
			componentsSUM = 0;

			delete [] neighboursSUM;
			neighboursSUM = 0;

			delete [] pathlengthsSUM_hops;
			pathlengthsSUM_hops = 0;

			delete [] pathlengthsSUM_meters;
			pathlengthsSUM_meters = 0;

			delete [] diametersSUM_hops;
			diametersSUM_hops = 0;

			delete [] diametersSUM_meters;
			diametersSUM_meters = 0;

			delete [] linksaddedSUM;
			linksaddedSUM = 0;

			delete [] linkslostSUM;
			linkslostSUM = 0;

			delete [] linksaddedTOTAL;
			linksaddedTOTAL = 0;

			delete [] linkslostTOTAL;
			linkslostTOTAL = 0;

			delete [] coveragepercentSUM;
			coveragepercentSUM = 0;

			delete [] clusteringSUM;
			clusteringSUM = 0;

			delete [] instabilitySUM;
			instabilitySUM = 0;


			for (int i = 0;i < max_amount;i++)
			{
				delete [] components[i];
				delete [] neighbours[i];
				delete [] links[i];
				delete [] pathlengths[i];
				delete [] diameters[i];
				delete [] coverage[i];
				delete [] clustering[i];
				delete [] instability[i];
			}
			delete [] components;
			delete [] neighbours;
			delete [] links;
			delete [] pathlengths;
			delete [] diameters;
			delete [] coverage;
			delete [] clustering;
			delete [] instability;
			delete [] predictability;
		}
		else
		{
			system("CLS");
			std::cout << "\n\nThe file path is not valid\n\n";
			std::system("PAUSE");
			system("CLS");
		}

	}	//end if
	

	return 0;

}	//end StatisticsSummary()

