//		Location Generator.cpp
//		Chris Briglio


#define NOMINMAX


#include "stdafx.h"

#include "convert.h"

#include <vector>

#include <cstdio>

#include <cstdlib>

#include <stdexcept>

#include <boost/lexical_cast.hpp>


//------------------------------------------------------------------------------FUNCTION HEADERS-------------------------------------------------------------------------------------//

//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


void NS2_output(std::string path);

void OPNET_output(std::string path);

void CORE_output(std::string path);

void transmission_refresh();

void run_time();

void xy_axis();

void globalsReset();


 //------------------------------------------------------------------------------GLOBAL VARIABLES-------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


static bool destination_reached[max_amount];

static double runtime = 0;

static double transmissionrange = 0;

static double refresh_rate  = 1;

static double events [max_amount][5];

static double destination [max_amount][4];

static double speed[max_amount];

static double events_speed[max_amount];

static double last_event_time = 0;

static int x_axis = 0,y_axis = 0;

std::string core_infilename2 = "";

bool exitflag = true;						//flag to control main function do-while loop


 //------------------------------------------------------------------------------LOCATION GENERATOR-----------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


using namespace boost;


/*
	LocationGenerator function:
		Takes in dynamic network simulation of NS2, CORE, or OPNET format and outputs the location of each node per refresh rate untill static.
		Promps user for file type (NS2,CORE,OPNET), gives option to do single or multiple files at once,
		promps for desired transmission range and refresh rate.
		Uses ' check*extension* ' functions to validate file type.
		Uses ' *filetype*_ouput ' functions for actual generation.
		Creates file folder for output in user choosen directory.
*/
int LocationGenerator(bool run)
{


//------------------------------------------------------------------------------VARIABLES--------------------------------------------------------------------------------------------//

//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//

#pragma region

	exitflag = false;						//flag to control main function do-while loop

	std::string searchextension = "";

	std::string path = "";

#pragma endregion

 //------------------------------------------------------------------------------PROGRAM----------------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


#pragma region
do			// main program do-while loop, insures program continues until user specifies exit (flag: exitflag)
{

	globalsReset();

	do
	{
		system("CLS");

		do 
		{		
  
			std::cout << "\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*             Location Generator              *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*     File types supported:                   *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*      NS2   (single file & multiple files)   *\n";
			std::cout << "\t*      OPNET (single file & multiple files)   *\n";
			std::cout << "\t*      CORE  (single file & multiple files)   *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*      Enter EXIT to exit to main menu        *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\n\n";
          
			std::cout << "\t          File Type: >> "; std::getline(std::cin,filetype);
		
 
			to_upper(filetype);

			if(filetype != "NS2" && filetype != "OPNET" && filetype != "CORE" && filetype != "EXIT")
			{
				system("CLS");
				std::cout << "The file type you have entered is unrecognizable, please try again.\n" << std::endl;
			}

		}while (filetype != "NS2" && filetype != "OPNET" && filetype != "CORE" && filetype != "EXIT");

		if (filetype != "EXIT")
		{
			if (filetype == "NS2")
				extension = ".txt";
			if (filetype == "OPNET")
				extension = ".xml";
			if (filetype == "CORE")
				extension = ".imn";

			system("CLS");

			do
			{

				//Ask for user to choose between single and multi-file intake

				std::cout << "\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*             Location Generator              *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*             Choose an option:               *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*              1. Single file                 *\n";
				std::cout << "\t*              2. Multiple files              *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t*        Enter EXIT to exit to menu           *\n";
				std::cout << "\t*                                             *\n";
				std::cout << "\t***********************************************\n";
				std::cout << "\n\n";

				std::cout << "\t                 >> "; std::getline(std::cin,inputmode);

				to_upper(inputmode);

				if (inputmode != "1" && inputmode != "2" && inputmode != "EXIT")
				{
					system("CLS");
					std::cout << "That option is not available, please choose a valid option,\n";
				}

			}while (inputmode != "1" && inputmode != "2"  && inputmode != "EXIT");

		}//end if

	}while(inputmode != "1" && inputmode != "2" && filetype != "EXIT");

#pragma endregion


//-------------------------------------------------------------------------------SINGLE FILE--------------------------------------------------------------------------------------//

//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


#pragma region
	if (inputmode == "1")
	{	

		system("CLS");

		do
		{

			std::cout << "\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*             Location Generator              *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*         Enter the name of the file          *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*         Enter EXIT to exit to menu          *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\n\n";

			std::cout << "\t                  >> "; std::getline (std::cin,infilename);		//Ask for the name of the file they want to use

			exitmask = infilename;

			to_upper(exitmask);

			if (checkEXT(infilename) == false && exitmask != "EXIT")
			{
				system("CLS");
				std::cout << "Unrecognizable file type.\n";
			}

		}while (checkEXT(infilename) == false && exitmask != "EXIT");

		if (exitmask != "EXIT")
		{

			system("CLS");

			txt_index = infilename.find(extension);

			if (fileExists(infilename) == true && filetype != "CORE")
			{

				transmission_refresh();
				if (refresh_rate != -1 && transmissionrange != -1)
				{
					if (run == true)
						run_time();
					else
						runtime = 0;

					if (runtime != -1)
					{

						if (filetype == "OPNET")
						{
							xy_axis();
							if (x_axis == -1 || y_axis == -1)
								return 0;
						}

						create_folder("");

						outfilename = infilename.erase(txt_index);
						std::string name = "";
						int nameLoc = outfilename.rfind("\\")+1; //Find where the file name itself starts (after the last backslash)
						name.insert(0, outfilename, nameLoc, (outfilename.length()-nameLoc)); //Add the file name to an empty string
						outfilename = name;
						outfilename.append("_out.txt");		//create output filename through input file name (add '_out' to end and folder path at beginning)
						outfilename.insert(0,folderPath);
						infilename.append(extension);		//set variables 'infilename' & 'outputfilename3'

						if (filetype == "NS2")
							NS2_output("");
						else if (filetype == "OPNET")
							OPNET_output("");
						else
						{
							std::cout << "Unrecognizable file type\n";
							system("PAUSE");
						}
						if (run == true)
							DynamicStatistics(outfilename,runtime,true);

					}//end if

				}//end if

			}
			else if (fileExists(infilename) == true)
			{

				core_infilename2 = infilename.erase(txt_index);
				core_infilename2.append(".scen");
				infilename.append(extension);

				if (fileExists(core_infilename2) == true)
				{

					system("CLS");

					transmission_refresh();
					if (refresh_rate != -1 && transmissionrange != -1)
					{

						if (run == true)
							run_time();
						else
							runtime = 0;
						if (runtime != -1)
						{

							folderPath = "";

							create_folder("");

							outfilename = infilename.erase(txt_index);

							outfilename.append("_out.txt");		//create output filename through input file name (add '_out' to end and folder path at beginning)

							outfilename.insert(0,folderPath);

							infilename.append(extension);		//set variables 'infilename' & 'outputfilename3'

							CORE_output("");

						}//end if

					}//end if
					
				}//end if
				else
				{
					system("CLS");
					std::cout << "The file " << core_infilename2 << " does not exist.\n\n";	
					system("PAUSE");
				}

			}
			else
			{
				system("CLS");
				std::cout << "The file " << infilename << " does not exist.\n\n";	
				system("PAUSE");
			}

		}//end if

	}	//end if 
#pragma endregion


//-------------------------------------------------------------------------------MULTIPLE FILE------------------------------------------------------------------------------------//

//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


#pragma region
	else if (inputmode == "2")
	{

		if (filetype == "NS2")
			searchextension = "\\*.txt";
		if (filetype == "OPNET")
			searchextension = "\\*.xml";
		if (filetype == "CORE")
			searchextension = "\\*.imn";

		system("CLS");

		std::string path = "";

		std::cout << "\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*             Location Generator              *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*       Enter file path to folder you         *\n";
		std::cout << "\t*                wish to use                  *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*        Enter EXIT to exit to menu           *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\n\n";

		std::cout << "\t                    >> "; std::getline(std::cin, path);

		exitmask = path;

		to_upper(exitmask);

		if (exitmask != "EXIT")
		{

			std::string fullpath = path + searchextension;

			WIN32_FIND_DATA FData;
			HANDLE hFind;

			hFind = FindFirstFile( fullpath.c_str(), &FData );

			if( hFind == INVALID_HANDLE_VALUE )
			{
				system("CLS");
				std::cout << "\n\nThe file path is not valid\n\n";
				std::system("PAUSE");
				system("CLS");
			}
			else
			{

				system("CLS");

				std::cout << "\nFile path " << path << " has been located.\n\n";


				path.append("\\");

				transmission_refresh();

				if (refresh_rate != -1 && transmissionrange != -1)
				{

					if (run == true)
						run_time();
					else
						runtime = 0;
					if (runtime != -1)
					{

						if (filetype == "OPNET")
						{
							xy_axis();
							if (x_axis == -1 || y_axis == -1)
								return 0;
						}

						do		//do-while loop: loops while files are still being located
						{

							std::string filePath = path + FData.cFileName;
							std::ifstream checkfile(filePath);
			
							if( checkfile.is_open() )		//if file has be found
							{

								infilename = FData.cFileName;

								txt_index = infilename.find(extension);

								if (filetype == "CORE")
								{
									core_infilename2 = infilename.erase(txt_index);
									core_infilename2.append(".scen");
									infilename.append(extension);
								}

								std::string fullfolderPath = "";		//create output folder

								folderPath = "";

								fullfolderPath.append(path);

								fullfolderPath.append(infilename.erase(txt_index));

								fullfolderPath.append(" output\\");

								create_folder(path);

								outfilename = infilename.erase(txt_index);

								outfilename.append("_out.txt");		//create output filename through input file name (add '_out' to end)

								fullfolderPath.append(outfilename);

								outfilename.insert(0,folderPath);

								infilename.append(extension);		//set variables 'infilename' & 'outputfilename3'

								if (filetype == "NS2")
									NS2_output(path);
								else if (filetype == "OPNET")
									OPNET_output(path);
								else if (filetype == "CORE")
									CORE_output(path);

								if (run == true)
									DynamicStatistics(fullfolderPath,runtime,true);

							}
							else
							{
								std::cout << "\n\nCould not open file " << FData.cFileName << "\n\n";
							}

						}while( FindNextFile(hFind, &FData) > 0 );
 
						if( GetLastError() != ERROR_NO_MORE_FILES )
						{
							std::cout << "\n\nERROR: did not reach end of files\n\n";
							std::cout << "Last file read " << FData.cFileName << "\n\n";
							std::system("PAUSE");
						}
						
					}//end if

				}//end if

			}// end if

		}//end if

	}	// end if
#pragma endregion


//-------------------------------------------------------------------------------END------------------------------------------------------------------------------------//

//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//

	else if (filetype == "EXIT" || inputmode == "EXIT")
	{
		exitflag = true;
	}

}while (exitflag == false);

return 0;

}	// end LocationGenerator()


 //------------------------------------------------------------------------------FUNCTIONS--------------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


/*
	NS2_output function:
	Reads in information from NS2 format dynamic network scenario file and outputs locations per second of all nodes
	until they reach their destination
	Argument: (path) used when user chooses folders of files option (multiple files), gives path to folder. input "" for single file
*/
void NS2_output(std::string path)
{

	std::string fullinfilename = path + infilename;
	std::string fulloutfilename = path + outfilename;

	num_nodes = 0;

	std::ifstream myinfile;
	myinfile.open(fullinfilename);

	if (myinfile.is_open())
	{

		//Declare every variable	

		int i = 0, k = 0, m = 0, o = 0,  count2 = 0, maxcount = 0, maxdist = 0, minhops = 1;

		double count1 = 0, sum = 0, count3 = 0, sum2 = 0, sumofhops = 0, maxconnected = 0;

		std::string line = "";

		double axis_x = 0, axis_y = 0;

		for (i = 0; i < max_amount; i++)		//resets all arrays to null values
		{

			 destination_reached[i] = false;
			 speed[i] = 0;
			 events_speed[i] = 0;

		} //end for 

		for (i = 0; i < max_amount; i++)		//resets all double arrays to null values
		{
			for (int a = 0; a < 5;a++)
			{
				if (a < 4)
				{
					events [i][a] = 0;

					destination [i][a] = 0;
				}
				else
				{
					events [i][a] = 0;
				}

			}	//end for

		} //end for

		//reads in the max x & y and number of nodes

		myinfile.ignore(10);

		std::getline(myinfile,line);

		num_nodes = atoi(line.c_str());

		if (num_nodes < 0)	num_nodes *= -1;

		std::cout << "Nodes found: " << num_nodes << "\n";

		myinfile.ignore(9);

		myinfile.close();

		myinfile.open(fullinfilename);


		std::getline(myinfile,line);
		std::getline(myinfile,line);

		line.erase(0,line.find("max x:")+7);
		axis_x = convertToDouble(line.substr(0,line.find(",")));

		line.erase(0,line.find("max y:")+7);
		axis_y = convertToDouble(line);

		//Create arrays for the x y and z coordinates

		double* x = new double[num_nodes];
		std::fill_n(x,num_nodes,0.0);

		double* y = new double[num_nodes];
		std::fill_n(y,num_nodes,0.0);

		double* z = new double[num_nodes];
		std::fill_n(z,num_nodes,0.0);

		int* slow_nodes = new int[num_nodes];		//array to keep track of nodes with small event time (0 if not, 1 if slow event time)
		std::fill_n(slow_nodes,num_nodes,0);
			
		int nodes = num_nodes;

		int num_events = 0;

		myinfile.close();

		myinfile.open(fullinfilename);


 //------------------------------------------------------------------------------STATIC--------------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//
		
		//Read through the file to find coordinates of each and every node

		while ( myinfile.good() )
		{

			std::getline(myinfile,line);

			if (line == "#") continue;

			std::cout << "\nBeginning to read simulation from " << infilename << "...\n";

			for (i = 0; i < num_nodes; i++) 
			{

				if (i < 10) 
				{

					myinfile.ignore(18);

					myinfile >> x[i];

					myinfile.ignore(18);

					myinfile >> y[i];

					myinfile.ignore(18);

					myinfile >> z[i];

				} 
				else if (i >= 10) 
				{

					myinfile.ignore(19);

					myinfile >> x[i];

					myinfile.ignore(19);

					myinfile >> y[i];

					myinfile.ignore(19);

					myinfile >> z[i];

				} 
				else if (i >= 100) 
				{ 

					myinfile.ignore(20);

					myinfile >> x[i];

					myinfile.ignore(20);

					myinfile >> y[i];

					myinfile.ignore(20);

					myinfile >> z[i];

				}	//end if

			std::cout << "Node " << i << " x: " << x[i] << " y: " << y[i] << " z: " << z[i] << "\n";

			}	//end for

			i = 0;

			int j = 0;

			double temp = 0;

			std::string god = "$god_";

			std::string ns = "$ns_";

			std::string numsign = "#";

			std::string node = "$node_";

			std::string temp_string = "";

		  	  
			//reads and discards the first line

			myinfile >> temp_string;

			while (temp_string != god && temp_string != numsign) 
			{

				//reads in the events at time 0

				//these are the initial destinations

				myinfile.ignore(4);

				myinfile >> temp;

				events[i][0] = temp;	//time

				myinfile.ignore(9);

				myinfile >> temp;

				events[i][1] = temp;	//node

				myinfile.ignore(10);

				myinfile >> temp;

				events[i][2] = temp;	//x

				myinfile.ignore(1);

				myinfile >> temp;

				events[i][3] = temp;	//y

				myinfile.ignore(1);

				myinfile >> temp;

				events[i][4] = temp;	//speed

				events_speed[i] = temp;

				std::cout << "i " << i << " node time " << events[i][0] << " node " << events[i][1] << " x " << events[i][2] << " y " << events[i][3] << " speed " << events[i][4] << " speed " << events_speed[i] << "\n";

				if (events_speed[i] < 0.0001)
				{
					slow_nodes[i] = 1;
					std::cout << "\nNode " << i << " has very slow event speed (" << events_speed[i] << ")\nThis will cause the program to run extremly slowly\n\n";
				}

				myinfile.ignore(1);

				myinfile >> temp_string;

				if (temp_string == "0")
				{

					std::cout << "destination array out of scope: " << j << "\n\n";
					std::system("PAUSE");

				}

				//used to determine the minimum running time for each node. This ensures that no node stops before its location is changed

				if (events[i][0] >= last_event_time) 
				{

					last_event_time = events[i][0];

				} //end if 

			
				//sets the destinations of each node

				destination[j][0] = events[i][1];

				destination[j][1] = events[i][2];

				destination[j][2] = events[i][3];

				destination[j][3] = events_speed[i];

				speed[j] = events[i][4];

				j++;			  

				i++;

	
				if (j >= max_amount)
				{
					std::cout << "\n\narray out of scope " << j << "\n\n";
					std::system("PAUSE");
				}


			} //end while  

			myinfile.ignore(26); 


 //------------------------------------------------------------------------------DYNAMIC--------------------------------------------------------------------------------------------//

 //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


			//now reads in the events that happen after 0s

			for (;;) 
			{

				myinfile >> temp_string;


				if (temp_string == god) 
				{

					myinfile.ignore(15);

				} 
				else if (temp_string == ns) 
				{

					myinfile.ignore(4);

					myinfile >> temp;

					myinfile.ignore(2);

					myinfile >> temp_string;	  



					if (temp_string == god) 
					{

						myinfile.ignore(17);

					} 
					else 
					{					  

						events[i][0] = temp;		//event time


						temp_string = temp_string.substr(7,3);

						const char * a = temp_string.c_str();

						events[i][1] = atof(a);		//node numer


						myinfile.ignore(9);

						myinfile >> temp;

						events[i][2] = temp;		//x

						myinfile.ignore(1);

						myinfile >> temp;

						events[i][3] = temp;		//y

						myinfile.ignore(1);

						myinfile >> temp;

						events[i][4] = temp;		//speed

						events_speed[i] = temp;

						myinfile.ignore(1);

						// std::cout << "i " << i << " node time " << events[i][0] << " node " << events[i][1] << " x " << events[i][2] << " y " << events[i][3] << " speed " << events[i][4] << " speed " << events_speed[i] << "\n";


						if (events[i][0] >= last_event_time) 
						{

							last_event_time = events[i][0];

						} //end if 

						i++;

					} //end if 

				} 
				else if (temp_string == numsign) 
				{

					int l = 0;

					num_events = i;

					for ( ;i < max_amount; i++)		//<= - <
					{

						for (int l = 0; l <= 4; l++)
						{
						
							//std::cout << "12 " << " speed " << events[12][4] << "\n";
							//std::cout << "i " << i << " speed " << events[i][4] << "\n";
							//std::cout << i << " " << l;
							//system("PAUSE");
								events[i][l] = 0;

							//std::cout << "12 " << " speed " << events[12][4] << "\n";
							//std::cout << "i " << i << " speed " << events[i][4] << "\n";
							//system("PAUSE");
							
						} //end for

					} //end for
					
					break;

				} //end if 

			} //end for


			break;

		} //end while 

			//for (int b = 0; b < 13;b++)
			//std::cout << "b " << b << " node time " << events[b][0] << " node " << events[b][1] << " x " << events[b][2] << " y " << events[b][3] << " speed " << events[b][4] << " speed " << events_speed[b] << "\n";
			//system("PAUSE");

		myinfile.close();

		std::ofstream myoutfile;

		std::cout << "Generating output file...\n";

		//writes initial information to the output file

		myoutfile.open(fulloutfilename);

		myoutfile << infilename << "\n";

		//writes 1 at the end to indicate troubleshoot mode and 0 for regular mode
		
				//|CHECK| - always is regular mode/ no troubleshoot?

		myoutfile << num_nodes << " " << refresh_rate << " " << transmissionrange << " " << axis_x << " " << axis_y << " 0\n";

									

		int node_number = 0;

		double end_time = 0;

		int tick = 0;

		int last_tick = 0;

		int timer = 0;

		std::string a = "";

		double x_speed = 0;

		double y_speed = 0;

		bool x_reached = false;

		bool y_reached = false;

		bool destination_changed = true;

		bool first_run = true;

		double distance = 0;

		double time = 0;


		//runs the location simulaton for each node, one at a time

		std::cout << "Starting location simulation...\n";

		for (node_number = 0; node_number < num_nodes; node_number++) 
		{

			tick = 0;

			last_tick = 0;

			timer = 0;

			a = "";

			x_speed = 0;

			y_speed = 0;

			x_reached = false;

			y_reached = false;

			destination_changed = true;

			first_run = true;

			distance = 0;

			time = 0;
			

			for (timer = 0;; timer++)
			{

				if (destination_changed == true) 
				{

					destination_changed = false;

					x_reached = false;

					y_reached = false;

					destination_reached[node_number] = false;

					// determines speed and distance to the destination

					distance = sqrt(sqr(destination[node_number][2] - y[node_number]) + sqr(destination[node_number][1] - x[node_number]));


					if(first_run == true) 
					{

						first_run = false;

						time = distance/events_speed[node_number];

					} 
					else 
					{

						if (destination[node_number][3] != 0)
						{
							time = distance/destination[node_number][3];
						}
						else
						{
							time = 0;
						}

					} //end if 

					if (time != 0)
					{

						x_speed = (destination[node_number][1] - x[node_number])/time;

						y_speed = (destination[node_number][2] - y[node_number])/time;

					}
					else
					{
						x_speed = 0;
						y_speed = 0;

					}

					if (x_speed == 0)
						x_reached = true;
					if (y_speed == 0)
						y_reached = true;

				} //end if 

				//std::cout << "Node: " << node_number << " timer: " << timer << " event time: " << events[node_number][0] << "\n";
				//std::cout << "x dest: " << destination[node_number][1] << " y dest: " << destination[node_number][2] << " distance: " << distance<< "\n";
				//std::cout << "x: " << x[node_number] << " y: " << y[node_number]<< "\n";
				//std::cout << "time: " << time << " speed: " << destination[node_number][3] << " event speed: " << events_speed[node_number] << "\n";
				//std::cout << "x speed: " << x_speed << " y speed: " << y_speed << "\n";
				//system("PAUSE");
				//system("CLS");

				//every 1 second, the nodes are moved at a determined speed and direction

				if (timer - tick >= 1) 
				{

					last_tick = tick;

					tick = timer;	

					//determines if node has reached its destination		
					
					//|CHECK| - assumes if destination is reached in x direction auto puts it to destination in y direction

					if (x_reached == false && x_speed > 0 && (x[node_number] + x_speed) >= destination[node_number][1]) 
					{

						x_reached = true;

						x[node_number] = destination[node_number][1];

					} 
					else if (x_reached == false && x_speed < 0 && (x[node_number] + x_speed) <= destination[node_number][1]) 
					{

						x_reached = true;

						x[node_number] = destination[node_number][1]; 

					} 
					else
					{

						if (x_reached == false)		x[node_number] += x_speed;

					} //end if 
					
					if (y_reached == false && y_speed < 0 && (y[node_number] + y_speed) <= destination[node_number][2]) 
					{

						y_reached = true;

						y[node_number] = destination[node_number][2];

					}
					else if (y_reached == false && y_speed > 0 && (y[node_number] + y_speed) >= destination[node_number][2]) 
					{

						y_reached = true;

						y[node_number] = destination[node_number][2];

					}
					else
					{

						if (y_reached == false)		y[node_number] += y_speed;

					} //end if 

					if (x_reached == true && y_reached == true)
					{
						destination_reached[node_number] = true;
					}


					//writes the node number, time and location to file

					myoutfile << node_number << " " << timer << " " << x[node_number] << " " << y[node_number] << "\n";

					for (int i = 0; i <= max_amount; i++) 
					{					

						if (events[i][0] > last_tick && events[i][0] <= tick) 
						{

							//changes destination if an event has occured at this time

							if (events[i][1] == node_number) 
							{

								destination[node_number][0] = events[i][1];	//node

								destination[node_number][1] = events[i][2];	//x

								destination[node_number][2] = events[i][3];	//y

								destination[node_number][3] = events[i][4];	//speed

								//std::cout << "i " << i << " node time " << events[i][0] << " node " << events[i][1] << " x " << events[i][2] << " y " << events[i][3] << " speed " << events[i][4] << " event speed: " << events_speed[node_number] << "\n\n";

								//std::cout << "Node: " << node_number << " timer: " << timer << " new event time: " << events[i][0] << "\n";
								//std::cout << "new x dest: " << destination[node_number][1] << " new y dest: " << destination[node_number][2] << "\n";
								//std::cout << "last x: " << x[node_number] << " last y: " << y[node_number] << "\n";
								//std::cout << "new speed: " << destination[node_number][3] << "\n";
								//system("PAUSE");
								//system("CLS");

								destination_changed = true;

							} //end if 

						} //end if

					} //end for

				}// end if 			

				//if the node has reached its destination and all events have passed, the loop is stopped

				if (destination_reached[node_number] == true && timer >= last_event_time + 1 && destination_changed != true)
				{

					if (timer >= end_time) 
					{

						end_time = timer;

					} //end if 

					break;

				} //end if 

			} //end for

			myoutfile << "Node " << node_number << " has reached its destination.\n\n";

			system("CLS");

			std::cout << infilename << "\n";
			std::cout << "Running location simulation...\n";

			for (int g = 0; g < num_nodes; g++)
			{
				if (slow_nodes[g] == 1)
				{
					std::cout << "Node " << g << " has very slow event speed (" << events_speed[g] << ")\n";
				}
			}

			std::cout << node_number+1 << "/" << num_nodes << " complete\n";

		} //end for

		myoutfile <<"#\n" << end_time;

		myoutfile.close();

		remove("destination.txt");

		system("CLS");

		std::cout << "\nThe location information has been successfully written to:\n" << fulloutfilename << "\n\n";

		//deletes pointers

		delete [] x;
		x = 0;

		delete [] y;
		y = 0;

		delete [] z;
		z = 0;

		delete [] slow_nodes;
		slow_nodes = 0;

	}	// end if
	else 
	{  

		std::cout << "\n\nUnable to open file: " << fullinfilename << "\n\n" ;

	} //end if 

	if (myinfile.is_open())
		myinfile.close();

}	// end of function


/*
	OPNET_output function:
		Reads in dynamic network scenarios.
		First reads through the initialization file (.xml) to find the # of nodes and the initial coordinates.
		Then goes through user specified folder for all trajectory files and calculates node movement one node at
		a time. Determines next file to open by using a Generic File Name. ex:
		Sample file:		 NuNodes 10-15nodes_RECOVERED9-Enterprise Network.1.trj
		Generic File Name:   NuNodes 10-15nodes_RECOVERED9-Enterprise Network.
		# before '.trj' represents the node whose trajectories are within the file.
		Reads through starting at node 1 by continuously increasing the number before the '.trj'
		by increments of 1.
		Outputs locations per second of all nodes until they reach their destination		
*/
void OPNET_output(std::string path)
{

	int j = 0;

	for (int i = 0; i < max_amount; i++)		//resets all arrays to null values
	{
		destination_reached[i] = false;
		speed[i] = 0;
		events_speed[i] = 0;
	} //end for 
	for (int i = 0; i < max_amount; i++)		//resets all double arrays to null values
	{
		for (int a = 0; a < 4;a++)
		{
			if (a != 4)
			{
				events [i][a] = 0;
				destination [i][a] = 0;
			}
			else
			{
				events [i][a] = 0;
			}
		}	//end for
	} //end for

	num_nodes = 0;

	std::string line;
	std::string::size_type w;

	std::string fullinfilename = path + infilename;

	bool correctnode = true;

	//Read through the input file to find how many nodes the network has. 
    std::ifstream myinfile (fullinfilename);

	if (myinfile.is_open())
	{

		system("CLS");
	
		std::cout << "Beginning to read Initial setup...\n";

		while (myinfile.good())			//goes through file and find total # of nodes
		{

			correctnode = true;

			getline (myinfile, line);
			if (line.find("		<node name=\"") != std::string::npos) 
			{
				
				line.erase(0,line.find("\"")+1);

				line = line.substr(0,line.find("\""));

				for (size_t i = 0; i < line.length() && correctnode == true;i++)
				{
					if (line[i] > '9' || line[i] < '0')
					{
						correctnode = false;
					}
				}

				if (correctnode == true)
					num_nodes++;

			}//end if

		}//end while


		myinfile.close();

		std::cout << "Nodes found: " << num_nodes << "\n";

		myinfile.open(fullinfilename);

		int end_time = 0;

		//Create arrays for the x y coordinates of each node

		double* x = new double[num_nodes];
		std::fill_n(x,num_nodes,0.0);

		double* y = new double[num_nodes];
		std::fill_n(y,num_nodes,0.0);

		bool slow_node = false;

		//Read through the input file for the initial coordinates of each node (input file = .xml)

		while ( myinfile.good() )
		{

			getline(myinfile,line);

			if (line.find ("</ext-attr>") != std::string::npos) 
			{	

				getline(myinfile,line);

				if (line.find("<attr name=\"x position\"")  != std::string::npos)
				{

					//x coordinate

					line.erase(0,34);

					w = line.find('"', 0);

					line.erase(w,std::string::npos);

					x[j] = convertToDouble(line);

					//y coordinate

					getline(myinfile,line);

					line.erase(0,34);

					w = line.find('"', 0);

					line.erase(w,std::string::npos);

					y[j] = convertToDouble(line);

					j++;
					continue;

				}//end if

			}	//end if
			else 
			{
				continue;
			}

		}	//end while

		myinfile.close();


 //------------------------------------------------------------------------------DYNAMIC--------------------------------------------------------------------------------------------//

 //---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//


		std::string fulloutfilename = path + outfilename;

		std::ofstream myoutfile;

		//writes initial information to the output file

		myoutfile.open(fulloutfilename);

		myoutfile << infilename << "\n";

		//writes 1 at the end to indicate troubleshoot mode and 0 for regular mode

		myoutfile << num_nodes << " " << refresh_rate << " " << transmissionrange << " " << x_axis << " " << y_axis << " 0\n";		//put 0 0 as axis not specified

		int txt_index = 0;

		int trj_num = 0;	//node that is being worked on

		int dest_time = 0;	//time when node reaches its destination

		int dest_change = 0;	//stores the amount of times the destination will be changed (called Coordinate_count in file)

		double x_speed = 0;
		double y_speed = 0;

		bool x_reached = false;
		bool y_reached = false;

		double distance = 0;

		int tick = 0;

		int last_tick = 0;

		int timer = 0;

		double time = 0;

		std::string temp = "";

		std::string fileGenericName = "";
		std::string GenricNameExtension = "";	//#.trj

		txt_index = infilename.find(".xml");

		std::string trj_path = path + infilename.erase(txt_index);

		std::string extension = "\\*.trj";

		std::string fulltrjpath = trj_path + extension;

		WIN32_FIND_DATA FData;
		HANDLE hFind;

		hFind = FindFirstFile( fulltrjpath.c_str(), &FData );

		if( hFind == INVALID_HANDLE_VALUE )
		{

			std::cout << "\n\nThe file path is not valid\n\n";
			std::system("PAUSE");
			return;
		}
		else
		{

			std::cout << "\nFile path " << trj_path << " has been located.\n\n";

			trj_path.append("\\");

			fileGenericName = FData.cFileName;

			fileGenericName.erase(fileGenericName.find(".")+1,std::string::npos);

			std::cout << "Starting location simulation...\n";
		
			do
			{

				std::string filePath = trj_path + FData.cFileName;
				myinfile.open(filePath);


				distance = 0;

				tick = 0;

				last_tick = 0;

				timer = 0;

				time = 0;

				x_speed = 0;
				y_speed = 0;

			
				if( myinfile.is_open() )		//if file has be found
				{

					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');		//ignore first 6 lines
					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
					
					getline(myinfile,temp);
					temp.erase(0,temp.find(" "));

					dest_change = atoi(temp.c_str()) - 1;		//find out how many times the destination will change

					if (dest_change > 10000)
						std::cout << "Curent node changes its destination " << dest_change << " times.\nThis may cause the program to run slowly\n";

					myinfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');


					//Get the destination coordinates and time it takes to reach the destination

					for (int g = 0;g < dest_change;g++)
					{

						std::getline(myinfile,temp);

						events[trj_num][0] = convertToDouble(temp.substr(0,temp.find(" ")));	//x destination

						if (_isnan(events[trj_num][0]) != 0 || _finite(events[trj_num][0]) == 0)
						{
							std::cout << "ERROR converting x destination from file\n";
							system("PAUSE");
							myinfile.close();
							myoutfile.close();
							return;
						}

						temp.erase(0,temp.find(",")+1);

						events[trj_num][1] = convertToDouble(temp.substr(0,temp.find(" ")));	//y destination

						if (_isnan(events[trj_num][1]) != 0 || _finite(events[trj_num][1]) == 0)
						{
							std::cout << "ERROR converting y destination from file\n";
							system("PAUSE");
							myinfile.close();
							myoutfile.close();
							return;
						}

						temp.erase(0,temp.find(",")+1);
						temp.erase(0,temp.find(",")+1);
					
						txt_index = temp.find('s');

						temp.erase(txt_index,std::string::npos);

						events[trj_num][2] = convertToDouble(temp.substr(0,temp.find('h')+1)) * 120;	//adds the hours and makes them into seconds by * 120

						temp.erase(0,temp.find('h')+1);

						events[trj_num][2] += convertToDouble(temp.substr(0,temp.find('m')+1)) * 60;	//adds the minutes and makes them into seconds by * 60

						temp.erase(0,temp.find('m')+1);

						events[trj_num][2] += convertToDouble(temp);									//adds the seconds

						if (_isnan(events[trj_num][2]) != 0 || _finite(events[trj_num][2]) == 0)
						{
							std::cout << "ERROR converting event time from file\n";
							system("PAUSE");
							myinfile.close();
							myoutfile.close();
							return;
						}

						//std::cout << "x: " << events[trj_num][0] << " y: " << events[trj_num][1] << " t: " << events[trj_num][2] << "\n"; //OUTPUT

						//generate node movement untill destination is reached

						x_reached = false;
						y_reached = false;

						destination_reached[trj_num] = false;


						distance = sqrt(sqr(events[trj_num][1] - y[trj_num]) + sqr(events[trj_num][0] - x[trj_num]));	//only calculate when timer = 0 ? (take out of loop)

						if (_isnan(distance) != 0 || _finite(distance) == 0)
						{
							std::cout << "ERROR calculating distance to destination for node: " << trj_num << "\n";
							system("PAUSE");
							myinfile.close();
							myoutfile.close();
							return;
						}

						//time = distance/events[trj_num][2];	//Interprets events[trj_num][2] as speed (this may be time)
						time = events[trj_num][2];	//Interprets events[trj_num][2] as time	***TESTING***

						if (_isnan(time) != 0 || _finite(time) == 0)
						{
							std::cout << "ERROR calculating time to destination for node: " << trj_num << "\n";
							system("PAUSE");
							myinfile.close();
							myoutfile.close();
							return;
						}

						if (time != 0)
						{

							x_speed = (events[trj_num][0] - x[trj_num])/time;

							if (_isnan(x_speed) != 0 || _finite(x_speed) == 0)
							{
								std::cout << "ERROR calculating speed in x direction for node: " << trj_num << "\n";
								system("PAUSE");
								myinfile.close();
								myoutfile.close();
								return;
							}

							y_speed = (events[trj_num][1] - y[trj_num])/time;

							if (_isnan(y_speed) != 0 || _finite(y_speed) == 0)
							{
								std::cout << "ERROR calculating speed in y direction for node: " << trj_num << "\n";
								system("PAUSE");
								myinfile.close();
								myoutfile.close();
								return;
							}

						}
						else
						{
							x_speed = 0;
							y_speed = 0;
						}
							

						if (x_speed == 0)
							x_reached = true;
						if (y_speed == 0)
							y_reached = true;


						for (timer = 0;destination_reached[trj_num] == false; timer++)
						{

							//every 1 second, the nodes are moved at a determined speed and direction

							if (timer - tick >= 1) 
							{

								last_tick = tick;

								tick = timer;	

								//determines if node has reached its destination		

								if (x_reached == false && x_speed > 0 && (x[trj_num] + x_speed) >= events[trj_num][0]) 
								{

									x_reached = true;

									x[trj_num] = events[trj_num][0];

								} 
								else if (x_reached == false && x_speed < 0 && (x[trj_num] + x_speed) <= events[trj_num][0]) 
								{

									x_reached = true;

									x[trj_num] = events[trj_num][0]; 

								} 
								else
								{

									if (x_reached == false)		x[trj_num] += x_speed;

								} //end if 
					
								if (y_reached == false && y_speed < 0 && (y[trj_num] + y_speed) <= events[trj_num][1]) 
								{

									y_reached = true;

									y[trj_num] = events[trj_num][1];

								}
								else if (y_reached == false && y_speed > 0 && (y[trj_num] + y_speed) >= events[trj_num][1]) 
								{

									y_reached = true;

									y[trj_num] = events[trj_num][1];

								}
								else
								{

									if (y_reached == false)		y[trj_num] += y_speed;

								} //end if 

								if (x_reached == true && y_reached == true)
								{
									destination_reached[trj_num] = true;
								}

								//writes the node number, time and location to file

								myoutfile << trj_num << " " << timer << " " << x[trj_num] << " " << y[trj_num] << "\n";

							}// end if 			

						} //end for

						if (timer >= end_time) 
						{

							end_time = timer;

						} //end if 

					//std::cout << "Distance: " << distance << " time: " << time << "x speed: " << x_speed << " y speed: " << y_speed << " timer: " << timer << " tick: " << tick  << "\n";	//OUTPUT
					//system("PAUSE");

					} //end for


					myoutfile << "Node " << trj_num << " has reached its destination.\n\n";

				}
				else
				{
					std::cout << "\n\nCould not open file " << FData.cFileName << "\n\n";
				}

				system("CLS");

				std::cout << infilename << "\n";
				std::cout << "Running location simulation...\n";

				std::cout << trj_num+1 << "/" << num_nodes << " complete\n";

				trj_num++;

				if (trj_num < num_nodes)
				{

					GenricNameExtension = lexical_cast<std::string>(trj_num+1) + ".trj";

					fulltrjpath = trj_path + fileGenericName + GenricNameExtension;

					hFind = FindFirstFile( fulltrjpath.c_str() , &FData );

					if( hFind == INVALID_HANDLE_VALUE )
					{

						std::cout << "\n\nThe file path is not valid\n\n";
						std::system("PAUSE");
        
					}//end if

				}//end if

				myinfile.close();

			}while (trj_num < num_nodes);



		myoutfile <<"#\n" << end_time;

		remove("destination.txt");

		system("CLS");

		std::cout << "\nThe location information has been successfully written to:\n" << fulloutfilename << "\n\n";

		}	//end if

		myoutfile.close();

		//deletes pointers

		delete [] x;
		x = 0;

		delete [] y;
		y = 0;


	}//end if
	else
	{
		std::cout << "Unable to open file:\n" << fullinfilename << "\n";
		system("PAUSE");
	}

}	//function end


/*
	CORE_output function:
		Reads in CORE dynamic network scenarios
		First reads setup file to find number of mobile nodes (static nodes are not included)
		Node is dynamic when its 'model' is 'mdr' and not 'router'
		Then reads in the scenario file, confirms scenario by comparing the amount of nodes.
		Then outputs locations per second of all nodes until they reach their destination
*/
void CORE_output(std::string path)
{

 //------------------------------------------------------------------------------STATIC--------------------------------------------------------------------------------------------//

 //---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//

	std::string fullinfilename = path + infilename;

	num_nodes = 0;

	//Read through the file to see how many nodes there are

	std::ifstream infile (fullinfilename);

	if (infile.is_open())
	{

		std::string line = "";

		std::string ns = "$ns_";

		std::string numsign = "#";

		std::string node = "$node_";

		int nodecheck = 0;

		double max_time = 0;

		double max_x = 0;

		double max_y = 0;
		
		std::cout << "Beginning to read files...\n";

		while(infile.good()) 
		{

			getline(infile, line);

			if (line.find("    type router") != std::string::npos)	//if a node
			{
				getline(infile, line);

				if (line.find("    model router") == std::string::npos)		//if not static
				{
					num_nodes++;
				}
			}

		}//end while

		infile.close();

		//Create arrays for the x y & z coordinates of each nodes
  		double* x = new double[num_nodes];
		std::fill_n(x,num_nodes,0.0);

		double* y = new double[num_nodes];
		std::fill_n(y,num_nodes,0.0);

		double* z = new double[num_nodes];
		std::fill_n(z,num_nodes,0.0);


 //------------------------------------------------------------------------------DYNAMIC--------------------------------------------------------------------------------------------//

 //---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------//

		std::string fullinfilename2 = path + core_infilename2;

		infile.open(fullinfilename2);

		if (infile.is_open())
		{

			getline(infile,line);

			line.erase(0,line.find(" ")+1);
			line.erase(0,line.find(" ")+1);

			nodecheck = atoi(line.substr(0,line.find(",")).c_str());	//read in # nodes to check with setup file

			if (nodecheck == num_nodes)
			{

				line.erase(0,line.find(":")+1);
				line.erase(0,line.find(" ")+1);

				max_time = convertToDouble(line.substr(0,line.find(",")));	//read in max time

				line.erase(0,line.find(":")+1);
				line.erase(0,line.find(" ")+1);

				max_x = convertToDouble(line.substr(0,line.find(",")));	//read in max x

				line.erase(0,line.find(":")+1);
				line.erase(0,line.find(" ")+1);

				max_y = convertToDouble(line);	//read in max y

				do
				{
					getline(infile,line);
				}while (line.find(node) == std::string::npos);

				for (int i = 0; i < num_nodes; i++)			//read in initial x,y & z coordinates
				{

					line.erase(0,line.find("X")+1);
					line.erase(0,line.find(" ")+1);

					x[i] = convertToDouble(line);	//read in node 

					getline(infile,line);

					line.erase(0,line.find("Y")+1);
					line.erase(0,line.find(" ")+1);

					y[i] = convertToDouble(line);	//read in node 

					getline(infile,line);

					line.erase(0,line.find("Z")+1);
					line.erase(0,line.find(" ")+1);

					z[i] = convertToDouble(line);	//read in node 


					getline(infile,line);

				}	//end for

				do
				{
					getline(infile,line);
				}while (line.find(ns) == std::string::npos);

				int i = 0;

				while(line.find(ns) != std::string::npos)	//read in all events
				{

					line.erase(0,line.find("t")+1);
					line.erase(0,line.find(" ")+1);

					events[i][0] = convertToDouble(line.substr(0,line.find(" ")));	//read in event time

					line.erase(0,line.find("(")+1);

					events[i][1] = convertToDouble(line.substr(0,line.find(")")));	//read in node 

					line.erase(0,line.find("t")+1);
					line.erase(0,line.find(" ")+1);

					events[i][2] = convertToDouble(line.substr(0,line.find(" ")));	//read in x destination

					line.erase(0,line.find(" ")+1);

					events[i][3] = convertToDouble(line.substr(0,line.find(" ")));	//read in y destination

					line.erase(0,line.find(" ")+1);
					 
					events_speed[i] = convertToDouble(line.substr(0,line.find("\"")));	//read in speed

					
					//sets the destinations of each node

					destination[i][0] = events[i][1];	//set ? (possible use as flag that event has already taken place) why use j?

					destination[i][1] = events[i][2];	//set x destination

					destination[i][2] = events[i][3];	//set y destination

					i++;

					getline(infile,line);

				}//end while

				std::string fulloutfilename = path + outfilename;

				std::ofstream myoutfile;

				std::cout << "Generating output file...\n";

				//writes initial information to the output file

				myoutfile.open(fulloutfilename);

				myoutfile << infilename << "\n";

				//writes 1 at the end to indicate troubleshoot mode and 0 for regular mode
		
						//|CHECK| - always is regular mode/ no troubleshoot?

				myoutfile << num_nodes << " " << refresh_rate << " " << transmissionrange << " " << max_x << " " << max_y << " 0\n";


				int node_number = 0;

				double end_time = 0;

				int tick = 0;

				int last_tick = 0;

				int timer = 0;

				std::string a = "";

				int curr_event = 0;

				double x_speed = 0;

				double y_speed = 0;

				bool x_reached = false;

				bool y_reached = false;

				bool destination_changed = true;

				bool first_run = true;

				double distance = 0;

				double time = 0;


				//runs the location simulaton for each node, one at a time

				system("CLS");

				std::cout << infilename << "\n";
				std::cout << "Running location simulation...\n";
				std::cout << node_number << "/" << num_nodes << "\n";

				for (node_number = 0; node_number < num_nodes; node_number++) 
				{

					tick = 0;

					last_tick = 0;

					timer = 0;

					a = "";

					x_speed = 0;

					y_speed = 0;

					x_reached = false;

					y_reached = false;

					destination_changed = true;

					first_run = true;

					distance = 0;

					time = 0;


					for (timer = 0;; timer++)
					{

						if (destination_changed == true) 
						{

							destination_changed = false;

							x_reached = false;

							y_reached = false;

							destination_reached[node_number] = false;

							// determines speed and distance to the destination

							distance = sqrt(sqr(destination[node_number][2] - y[node_number]) + sqr(destination[node_number][1] - x[node_number]));


							if(first_run == true) 
							{

								first_run = false;

								time = distance/events_speed[node_number];

							} 
							else 
							{

								if (destination[node_number][3] != 0)
								{
									time = distance/destination[node_number][3];
								}
								else
								{
									time = 0;
								}

							} //end if 

							if (time != 0)
							{

								x_speed = (destination[node_number][1] - x[node_number])/time;

								y_speed = (destination[node_number][2] - y[node_number])/time;

							}
							else
							{
								x_speed = 0;
								y_speed = 0;

							}

							if (x_speed == 0)
								x_reached = true;
							if (y_speed == 0)
								y_reached = true;


						} //end if 

						/*
						std::cout << "Node: " << node_number << " timer: " << timer << " event time: " << events[node_number][0] << "\n";
						std::cout << "x dest: " << destination[node_number][1] << " y dest: " << destination[node_number][2] << "\n";
						std::cout << "distance: " << distance << " y: " << y[node_number] << " x: " << x[node_number] << "\n";
						std::cout << "time: " << time << " speed: " << destination[node_number][3] << "\n";
						std::cout << "x speed: " << x_speed << " y speed: " << y_speed << "\n";
						system("PAUSE");
						system("CLS");
						*/

						//every 1 second, the nodes are moved at a determined speed and direction

						if (timer - tick >= 1) 
						{

							last_tick = tick;

							tick = timer;	

							//determines if node has reached its destination		
					
							//|CHECK| - assumes if destination is reached in x direction auto puts it to destination in y direction

							if (x_reached == false && x_speed > 0 && (x[node_number] + x_speed) >= destination[node_number][1]) 
							{

								x_reached = true;		//reached x position in + direction

								x[node_number] = destination[node_number][1];

							} 
							else if (x_reached == false && x_speed < 0 && (x[node_number] + x_speed) <= destination[node_number][1]) 
							{

								x_reached = true;		//reached x position in - direction

								x[node_number] = destination[node_number][1]; 

							} 
							else
							{

								if (x_reached == false)		x[node_number] += x_speed;

							} //end if 
					
							if (y_reached == false && y_speed < 0 && (y[node_number] + y_speed) <= destination[node_number][2]) 
							{

								y_reached = true;			//reached y position in - direction

								y[node_number] = destination[node_number][2];

							}
							else if (y_reached == false && y_speed > 0 && (y[node_number] + y_speed) >= destination[node_number][2]) 
							{

								y_reached = true;			//reached y position in + direction

								y[node_number] = destination[node_number][2];

							}
							else
							{

								if (y_reached == false)		y[node_number] += y_speed;

							} //end if 

							if (x_reached == true && y_reached == true)
							{
								destination_reached[node_number] = true;
							}


							//writes the node number, time and location to file

							myoutfile << node_number << " " << timer << " " << x[node_number] << " " << y[node_number] << "\n";
						
							
						}// end if 		


						for (int i = 0; i <= max_amount && destination_changed == false; i++) 
						{

							//std::cout << "Node: " << events[i][1] << " Node: " << node_number <<"\n";

							if (events[i][1] == node_number+1)				//if proper node
							{
								/*
								std::cout << "Node: " << events[i][1] << " timer: " << timer << " event time: " << events[i][0] << "\n";
								std::cout <<"Last tick: " << last_tick << " tick: " << tick << "\n";
								system("PAUSE");
								system("CLS");
								*/
								//changes destination if an event has occured at this time

								if (events[i][0] > last_tick && events[i][0] <= tick)	//if event happens at current time
								{

									destination[node_number][0] = events[i][1];  // ?

									destination[node_number][1] = events[i][2];	//set new x

									destination[node_number][2] = events[i][3];	//set new y

									destination[node_number][3] = events_speed[i];	//set new speed

									destination_changed = true;


									/*
									std::cout << "Node: " << node_number << " timer: " << timer << " new event time: " << events[i][0] << "\n";
									std::cout << "new x dest: " << destination[node_number][1] << " new y dest: " << destination[node_number][2] << "\n";
									std::cout << "last x: " << x[node_number] << " last y: " << y[node_number] << "\n";
									std::cout << "new speed: " << destination[node_number][3] << "\n";
									system("PAUSE");
									system("CLS");
									*/

									//destination_reached[node_number] = false;

								} //end if 

							} //end if

						} //end for



						//if the node has reached its destination and all events have passed, the loop is stopped

						if (destination_reached[node_number] == true && timer >= max_time + 1)
						{

							if (timer >= end_time) 
							{

								end_time = timer;

							} //end if 

							break;

						} //end if 

					} //end for

					myoutfile << "Node " << node_number << " has reached its destination.\n\n";

					system("CLS");

					std::cout << infilename << "\n";
					std::cout << "Running location simulation...\n";
					std::cout << node_number+1 << "/" << num_nodes << " complete\n";

				} //end for

				myoutfile <<"#\n" << end_time;

				myoutfile.close();

				remove("destination.txt");

				system("CLS");

				std::cout << "\nThe location information has been successfully written to:\n" << outfilename << "\n\n";

			}
			else
			{
				system("CLS");
				std::cout << "Trajectory file does not match setup file\n";
				std::cout << "Setup: " << num_nodes << " Trajectory: " << nodecheck << "\n";
			}

		}
		else
		{
			std::cout << "Unable to open file:\n" << core_infilename2 << "\n";
			system("PAUSE");
		}

		//deletes pointers

		delete [] x;
		x = 0;

		delete [] y;
		y = 0;

		delete [] z;
		z = 0;

		infile.close();

	}
	else
	{
		std::cout << "Unable to open file:\n" << infilename << "\n";
		system("PAUSE");
	}

}	//function end


/*
	transmission_refresh function:
		promps user to enter desired transmission range and refresh rate to be used when creating statistics.
		Intake as string then checks if all characters are numbers, then uses 'convertToDouble' function
		to set transmissionrange and refresh_rate to desired number.
		returns tranmission range or refresh rate as -1 if user desires to return to menu
*/
void transmission_refresh()
{
	
	std::string transmission = "";
	std::string refresh = "";

	bool correct = true;

	do
	{

		correct = true;

		//Ask user for the tranmission range.

		std::cout << "\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*             Location Generator              *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*    Enter transmisson range of the network   *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*        Enter EXIT to exit to menu           *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\n\n";

		std::cout << "\t                   >> "; getline(std::cin,transmission);

		exitmask = transmission;

		to_upper(exitmask);

		if (exitmask != "EXIT")
		{

			for (size_t i = 0; i < transmission.length() && correct == true;i++)
			{
				if (transmission[i] > '9' || transmission[i] < '0')
				{
					correct = false;
					std::system("CLS");
					std::cout << "Invalid input\n";
				}
			}


			if (correct == true)
			{

				transmissionrange = convertToDouble(transmission);
				if (transmissionrange < 1)
				{
					correct = false;
					std::system("CLS");
					std::cout << "transmission range must be a positive integer greater than or equal to 1\n";
				}

			}

		}
		else
			transmissionrange = -1;

	}while(correct == false);

	system("CLS");

	if (exitmask != "EXIT")
	{

		do
		{

			correct = true;

			//Ask user for refresh rate

			std::cout << "\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*             Location Generator              *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*  Enter the refresh rate of the statistics   *\n";
			std::cout << "\t*                <in seconds>                 *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*        Enter EXIT to exit to menu           *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\n\n";
				
			std::cout << "\t                    >> "; getline(std::cin,refresh); 

			exitmask = refresh;

			to_upper(exitmask);

			if (exitmask != "EXIT")
			{

				for (size_t i = 0; i < refresh.length() && correct == true;i++)
				{
					if (refresh[i] > '9' || refresh[i] < '0')
					{
						correct = false;
						std::system("CLS");
						std::cout << "Invalid input\n";
					}
				}

				if (correct == true)
				{

					refresh_rate = convertToDouble(refresh);

					if (refresh_rate < 1)
					{
						refresh_rate = 0;
						std::system("CLS");
						std::cout << "Refresh rate must be a positive integer greater than or equal to 1\n";
					}

				}

			}
			else
				refresh_rate = -1;

		}while (correct == false || refresh_rate == 0 && exitmask != "EXIT");

		system("CLS");

	}

}


/*
	run_time function:
		Gets user to enter desired run time for statistics
		returns -1 if user wants to exit to main menu
*/
void run_time()
{
	std::string runcheck = "";
	bool correct = true;

	runtime = 0;

	do 
	{
		correct = true;
		std::system("CLS");

		std::cout << "\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*             Location Generator              *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*      Please enter the desired run time      *\n";
		std::cout << "\t*  (Warning: Run times greater than 250,000   *\n";
		std::cout << "\t*         may cause program to crash)         *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*        Enter EXIT to exit to menu           *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\n";

		std::cout << "\t                 >> "; getline(std::cin,runcheck);

		exitmask = runcheck;

		to_upper(exitmask);

		if (exitmask != "EXIT")
		{

			for (size_t i = 0; i < runcheck.length() && correct == true;i++)
			{
				if (runcheck[i] > '9' || runcheck[i] < '0')
				{
					correct = false;
					std::system("CLS");
					std::cout << "Invalid input\n";
				}
			}


			if (correct == true)
			{
				runtime = convertToDouble(runcheck);

				if (runtime < 1) 
				{
					runtime = 0;
					std::cout << "Please enter a runtime greater than 0\n";
				} 
				else 
				{
					runtime = floor(runtime);
				} 

			}

		}
		else
			runtime = -1;

	}while(correct == false || runtime == 0 && exitmask != "EXIT");


	system("CLS");

}


/*
	xy_axis function:
		promps user to enter desired values for the x and y axis to be used in the matlab section of program.
		Intake as string then checks if all characters are numbers, then uses 'convertToInterger' function
		to set x and y axis to desired number.
		returns x_axis or y_axis as -1 if user desires to return to menu
*/
void xy_axis()
{
	
	std::string xstring = "";
	std::string ystring = "";

	bool correct = true;

	do
	{

		correct = true;

		//Ask user for the tranmission range.

		std::cout << "\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*             Location Generator              *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*          Enter the x axis limit             *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t*        Enter EXIT to exit to menu           *\n";
		std::cout << "\t*                                             *\n";
		std::cout << "\t***********************************************\n";
		std::cout << "\n\n";

		std::cout << "\t                   >> "; getline(std::cin,xstring);

		exitmask = xstring;

		to_upper(exitmask);

		if (exitmask != "EXIT")
		{

			for (size_t i = 0; i < xstring.length() && correct == true;i++)
			{
				if (xstring[i] > '9' || xstring[i] < '0')
				{
					correct = false;
					std::system("CLS");
					std::cout << "Invalid input\n";
				}
			}


			if (correct == true)
			{

				x_axis = convertToInterger(xstring);
				if (x_axis < 1)
				{
					correct = false;
					std::system("CLS");
					std::cout << "axis must be greater or equal to 1\n";
				}

			}

		}
		else
			x_axis = -1;

	}while(correct == false);

	system("CLS");

	if (exitmask != "EXIT")
	{

		do
		{

			correct = true;

			//Ask user for refresh rate

			std::cout << "\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*             Location Generator              *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*          Enter the y axis limit             *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t*        Enter EXIT to exit to menu           *\n";
			std::cout << "\t*                                             *\n";
			std::cout << "\t***********************************************\n";
			std::cout << "\n\n";
			
			std::cout << "\t                   >> "; getline(std::cin,ystring);

			exitmask = ystring;

			to_upper(exitmask);

			if (exitmask != "EXIT")
			{

				for (size_t i = 0; i < ystring.length() && correct == true;i++)
				{
					if (ystring[i] > '9' || ystring[i] < '0')
					{
						correct = false;
						std::system("CLS");
						std::cout << "Invalid input\n";
					}
				}


				if (correct == true)
				{

					y_axis = convertToInterger(ystring);
					if (y_axis < 1)
					{
						correct = false;
						std::system("CLS");
						std::cout << "axis must be greater or equal to 1\n";
					}

				}

			}
			else
				y_axis = -1;

		}while (correct == false && exitmask != "EXIT");

		system("CLS");

	}

}


/*
	globalsReset function:
		Resets all global variables to their original values
*/
void globalsReset()
{

	num_nodes = 0;

	txt_index = 0;

	transmissionrange = 0;

	refresh_rate  = 1;

	last_event_time = 0;

	infilename = "";

	core_infilename2 = "";

	outfilename = "";

	folderPath = "";

	filetype = "";

	inputmode = "";

	
	for (int i = 0; i < max_amount; i++)		//resets all arrays to null values
	{
		destination_reached[i] = false;
		speed[i] = 0;
		events_speed[i] = 0;
	} //end for 

	for (int i = 0; i < max_amount; i++)		//resets all double arrays to null values
	{
		for (int a = 0; a < 4;a++)
		{
			if (a != 4)
			{
				events [i][a] = 0;
				destination [i][a] = 0;
			}
			else
			{
				events [i][a] = 0;
			}
		}	//end for
	} //end for

}

